@@ -19,6 +19,7 @@
 import functools
 import itertools
 import math
+import operator
 import random
 import re
 import struct
@@ -26,8 +27,6 @@ import time
 
 import lz4.block
 import numpy
-import numpy.lib.recfunctions
-from scipy import ndimage
 import six
 
 
@@ -97,10 +96,9 @@ def combine_timeseries(ts1, ts2):
     :param ts: The timeseries to combine.
     :return: A new timeseries.
     """
-    _, index = numpy.unique(
-        numpy.append(ts1['timestamps'], ts2['timestamps']),
-        return_index=True)
-    return numpy.append(ts1, ts2)[index]
+    ts = numpy.concatenate((ts1, ts2))
+    _, index = numpy.unique(ts['timestamps'], return_index=True)
+    return ts[index]
 
 
 class GroupedTimeSeries(object):
@@ -125,29 +123,50 @@ class GroupedTimeSeries(object):
                                                  return_counts=True)
 
     def mean(self):
-        return self._scipy_aggregate(ndimage.mean)
+        series = self.sum()
+        series['values'] /= self.counts
+        return series
 
     def sum(self):
-        return self._scipy_aggregate(ndimage.sum)
+        return make_timeseries(self.tstamps, numpy.bincount(
+            numpy.repeat(numpy.arange(self.counts.size), self.counts),
+            weights=self._ts['values']))
 
     def min(self):
-        return self._scipy_aggregate(ndimage.minimum)
+        ordered = self._ts['values'].argsort()
+        uniq_inv = numpy.repeat(numpy.arange(self.counts.size), self.counts)
+        values = numpy.zeros(self.tstamps.size)
+        values[uniq_inv[ordered][::-1]] = self._ts['values'][ordered][::-1]
+        return make_timeseries(self.tstamps, values)
 
     def max(self):
-        return self._scipy_aggregate(ndimage.maximum)
+        ordered = self._ts['values'].argsort()
+        uniq_inv = numpy.repeat(numpy.arange(self.counts.size), self.counts)
+        values = numpy.zeros(self.tstamps.size)
+        values[uniq_inv[ordered]] = self._ts['values'][ordered]
+        return make_timeseries(self.tstamps, values)
 
     def median(self):
-        return self._scipy_aggregate(ndimage.median)
+        ordered = numpy.lexsort((self._ts['values'], self.indexes))
+        # TODO(gordc): can use np.divmod when centos supports numpy 1.13
+        mid_diff = numpy.floor_divide(self.counts, 2)
+        odd = numpy.mod(self.counts, 2)
+        mid_floor = (numpy.cumsum(self.counts) - 1) - mid_diff
+        mid_ceil = mid_floor + (odd + 1) % 2
+        return make_timeseries(
+            self.tstamps, (self._ts['values'][ordered][mid_floor] +
+                           self._ts['values'][ordered][mid_ceil]) / 2.0)
 
     def std(self):
-        # NOTE(sileht): ndimage.standard_deviation is really more performant
-        # but it use ddof=0, to get the same result as pandas we have to use
-        # ddof=1. If one day scipy allow to pass ddof, this should be changed.
-        return self._scipy_aggregate(ndimage.labeled_comprehension,
-                                     remove_unique=True,
-                                     func=functools.partial(numpy.std, ddof=1),
-                                     out_dtype='float64',
-                                     default=None)
+        mean_ts = self.mean()
+        diff_sq = numpy.square(self._ts['values'] -
+                               numpy.repeat(mean_ts['values'], self.counts))
+        bin_sum = numpy.bincount(
+            numpy.repeat(numpy.arange(self.counts.size), self.counts),
+            weights=diff_sq)
+        return make_timeseries(self.tstamps[self.counts > 1],
+                               numpy.sqrt(bin_sum[self.counts > 1] /
+                                          (self.counts[self.counts > 1] - 1)))
 
     def count(self):
         return make_timeseries(self.tstamps, self.counts)
@@ -155,36 +174,27 @@ class GroupedTimeSeries(object):
     def last(self):
         cumcounts = numpy.cumsum(self.counts) - 1
         values = self._ts['values'][cumcounts]
-
         return make_timeseries(self.tstamps, values)
 
     def first(self):
-        counts = numpy.insert(self.counts[:-1], 0, 0)
-        cumcounts = numpy.cumsum(counts)
+        cumcounts = numpy.cumsum(self.counts) - self.counts
         values = self._ts['values'][cumcounts]
         return make_timeseries(self.tstamps, values)
 
     def quantile(self, q):
-        return self._scipy_aggregate(ndimage.labeled_comprehension,
-                                     func=functools.partial(
-                                         numpy.percentile,
-                                         q=q,
-                                     ),
-                                     out_dtype='float64',
-                                     default=None)
-
-    def _scipy_aggregate(self, method, remove_unique=False, *args, **kwargs):
-        if remove_unique:
-            tstamps = self.tstamps[self.counts > 1]
-        else:
-            tstamps = self.tstamps
-
-        if len(tstamps) == 0:
-            return make_timeseries([], [])
-
-        values = method(self._ts['values'], self.indexes, tstamps,
-                        *args, **kwargs)
-        return make_timeseries(tstamps, values)
+        ordered = numpy.lexsort((self._ts['values'], self.indexes))
+        min_pos = numpy.cumsum(self.counts) - self.counts
+        real_pos = min_pos + (self.counts - 1) * (q / 100)
+        floor_pos = numpy.floor(real_pos).astype(numpy.int, copy=False)
+        ceil_pos = numpy.ceil(real_pos).astype(numpy.int, copy=False)
+        values = (
+            self._ts['values'][ordered][floor_pos] * (ceil_pos - real_pos) +
+            self._ts['values'][ordered][ceil_pos] * (real_pos - floor_pos))
+        # NOTE(gordc): above code doesn't compute proper value if pct lands on
+        # exact index, it sets it to 0. we need to set it properly here
+        exact_pos = numpy.equal(floor_pos, ceil_pos)
+        values[exact_pos] = self._ts['values'][ordered][floor_pos][exact_pos]
+        return make_timeseries(self.tstamps, values)
 
     def derived(self):
         timestamps = self._ts_for_derive['timestamps'][1:]
@@ -211,10 +221,6 @@ class TimeSerie(object):
     def from_data(cls, timestamps=None, values=None):
         return cls(make_timeseries(timestamps, values))
 
-    @classmethod
-    def from_tuples(cls, timestamps_values):
-        return cls.from_data(*zip(*timestamps_values))
-
     def __eq__(self, other):
         return (isinstance(other, TimeSerie) and
                 numpy.all(self.ts == other.ts))
@@ -357,19 +363,17 @@ class BoundTimeSerie(TimeSerie):
         except ValueError:
             raise InvalidData
 
-        timestamps = numpy.cumsum(timestamps)
-        timestamps = timestamps.astype(dtype='datetime64[ns]', copy=False)
-
         return cls.from_data(
-            timestamps,
+            numpy.cumsum(timestamps),
             values,
             block_size=block_size,
             back_window=back_window)
 
     def serialize(self):
         # NOTE(jd) Use a double delta encoding for timestamps
-        timestamps = numpy.insert(numpy.diff(self.timestamps), 0, self.first)
-        timestamps = timestamps.astype(dtype='<Q', copy=False)
+        timestamps = numpy.empty(self.timestamps.size, dtype='<Q')
+        timestamps[0] = self.first
+        timestamps[1:] = numpy.diff(self.timestamps)
         return self._compress(timestamps.tobytes() + self.values.tobytes())
 
     @classmethod
@@ -484,27 +488,26 @@ class SplitKey(object):
         return hash(str(self.key.astype('datetime64[ns]')) +
                     str(self.sampling.astype('timedelta64[ns]')))
 
-    def __lt__(self, other):
+    def _compare(self, op, other):
         if isinstance(other, SplitKey):
             if self.sampling != other.sampling:
                 raise TypeError(
                     "Cannot compare %s with different sampling" %
                     self.__class__.__name__)
-            return self.key < other.key
+            return op(self.key, other.key)
         if isinstance(other, numpy.datetime64):
-            return self.key < other
+            return op(self.key, other)
         raise TypeError("Cannot compare %r with %r" % (self, other))
 
+    def __lt__(self, other):
+        return self._compare(operator.lt, other)
+
     def __eq__(self, other):
-        if isinstance(other, SplitKey):
-            if self.sampling != other.sampling:
-                raise TypeError(
-                    "Cannot compare %s with different sampling" %
-                    self.__class__.__name__)
-            return self.key == other.key
-        if isinstance(other, numpy.datetime64):
-            return self.key == other
-        raise TypeError("Cannot compare %r with %r" % (self, other))
+        return self._compare(operator.eq, other)
+
+    def __ne__(self, other):
+        # neither total_ordering nor py2 sets ne as the opposite of eq
+        return self._compare(operator.ne, other)
 
     def __str__(self):
         return str(float(self))
@@ -526,8 +529,7 @@ class AggregatedTimeSerie(TimeSerie):
     COMPRESSED_SERIAL_LEN = struct.calcsize("<Hd")
     COMPRESSED_TIMESPAMP_LEN = struct.calcsize("<H")
 
-    def __init__(self, sampling, aggregation_method, ts=None, max_size=None,
-                 truncate=True):
+    def __init__(self, sampling, aggregation_method, ts=None):
         """A time serie that is downsampled.
 
         Used to represent the downsampled timeserie for a single
@@ -536,10 +538,7 @@ class AggregatedTimeSerie(TimeSerie):
         """
         super(AggregatedTimeSerie, self).__init__(ts)
         self.sampling = sampling
-        self.max_size = max_size
         self.aggregation_method = aggregation_method
-        if truncate:
-            self._truncate()
 
     def resample(self, sampling):
         return AggregatedTimeSerie.from_grouped_serie(
@@ -547,15 +546,10 @@ class AggregatedTimeSerie(TimeSerie):
 
     @classmethod
     def from_data(cls, sampling, aggregation_method, timestamps,
-                  values, max_size=None):
-        # TODO(gordc): we don't truncate here but we might want to.
-        # specifically, when we store_timeserie_split, we merge with existing
-        # serie but we don't truncate so we might merge stuff that is beyond
-        # policy (not visible because we follow policy and truncate on GET)
+                  values):
         return cls(sampling=sampling,
                    aggregation_method=aggregation_method,
-                   ts=make_timeseries(timestamps, values),
-                   max_size=max_size, truncate=False)
+                   ts=make_timeseries(timestamps, values))
 
     @staticmethod
     def _get_agg_method(aggregation_method):
@@ -570,6 +564,22 @@ class AggregatedTimeSerie(TimeSerie):
             aggregation_method_func_name = aggregation_method
         return aggregation_method_func_name, q
 
+    def truncate(self, oldest_point):
+        """Truncate the time series up to oldest_point excluded.
+
+        :param oldest_point: Oldest point to keep from, this excluded.
+        :type oldest_point: numpy.datetime64 or numpy.timedelta64
+        """
+        last = self.last
+        if last is None:
+            # There's nothing to truncate
+            return
+        if isinstance(oldest_point, numpy.timedelta64):
+            oldest_point = last - oldest_point
+        index = numpy.searchsorted(self.ts['timestamps'], oldest_point,
+                                   side='right')
+        self.ts = self.ts[index:]
+
     def split(self):
         # NOTE(sileht): We previously use groupby with
         # SplitKey.from_timestamp_and_sampling, but
@@ -589,38 +599,33 @@ class AggregatedTimeSerie(TimeSerie):
             start = end
 
     @classmethod
-    def from_timeseries(cls, timeseries, sampling, aggregation_method,
-                        max_size=None):
+    def from_timeseries(cls, timeseries, sampling, aggregation_method):
         # NOTE(gordc): Indices must be unique across all timeseries. Also,
         # timeseries should be a list that is ordered within list and series.
         if not timeseries:
             timeseries = [make_timeseries([], [])]
         return cls(sampling=sampling,
                    aggregation_method=aggregation_method,
-                   ts=numpy.concatenate(timeseries), max_size=max_size)
+                   ts=numpy.concatenate(timeseries))
 
     @classmethod
-    def from_grouped_serie(cls, grouped_serie, sampling, aggregation_method,
-                           max_size=None, truncate=False):
+    def from_grouped_serie(cls, grouped_serie, sampling, aggregation_method):
         agg_name, q = cls._get_agg_method(aggregation_method)
         return cls(sampling, aggregation_method,
                    ts=cls._resample_grouped(grouped_serie, agg_name,
-                                            q),
-                   max_size=max_size, truncate=truncate)
+                                            q))
 
     def __eq__(self, other):
         return (isinstance(other, AggregatedTimeSerie)
                 and super(AggregatedTimeSerie, self).__eq__(other)
-                and self.max_size == other.max_size
                 and self.sampling == other.sampling
                 and self.aggregation_method == other.aggregation_method)
 
     def __repr__(self):
-        return "<%s 0x%x sampling=%s max_size=%s agg_method=%s>" % (
+        return "<%s 0x%x sampling=%s agg_method=%s>" % (
             self.__class__.__name__,
             id(self),
             self.sampling,
-            self.max_size,
             self.aggregation_method,
         )
 
@@ -700,10 +705,9 @@ class AggregatedTimeSerie(TimeSerie):
         # initialize list to store alternating delimiter, float entries
         if compressed:
             # NOTE(jd) Use a double delta encoding for timestamps
-            timestamps = numpy.insert(
-                numpy.diff(self.timestamps) / offset_div,
-                0, (self.first - start.key) / offset_div)
-            timestamps = timestamps.astype('<H', copy=False)
+            timestamps = numpy.empty(self.timestamps.size, dtype='<H')
+            timestamps[0] = (self.first - start.key) / offset_div
+            timestamps[1:] = numpy.diff(self.timestamps) / offset_div
             payload = (timestamps.tobytes() + self.values.tobytes())
             return None, b"c" + self._compress(payload)
         # NOTE(gordc): this binary serializes series based on the split
@@ -718,30 +722,16 @@ class AggregatedTimeSerie(TimeSerie):
         first = self.first  # NOTE(jd) needed because faster
         e_offset = int((self.last - first) / offset_div) + 1
 
-        locs = numpy.cumsum(numpy.diff(self.timestamps)) / offset_div
-        locs = numpy.insert(locs, 0, 0)
-        locs = locs.astype(numpy.int, copy=False)
+        locs = numpy.zeros(self.timestamps.size, dtype=numpy.int)
+        locs[1:] = numpy.cumsum(numpy.diff(self.timestamps)) / offset_div
 
-        # Fill everything with zero
-        serial_dtype = [('b', '<?'), ('v', '<d')]
-        serial = numpy.zeros((e_offset,), dtype=serial_dtype)
+        # Fill everything with zero and set
+        serial = numpy.zeros((e_offset,), dtype=[('b', '<?'), ('v', '<d')])
+        serial['b'][locs] = numpy.ones_like(self.values, dtype='<?')
+        serial['v'][locs] = self.values
 
-        # Create a structured array with two dimensions
-        ones = numpy.ones_like(self.values, dtype='<?')
-        values = numpy.core.records.fromarrays((ones, self.values),
-                                               dtype=serial_dtype)
-
-        serial[locs] = values
-
-        payload = serial.tobytes()
         offset = int((first - start.key) / offset_div) * self.PADDED_SERIAL_LEN
-        return offset, payload
-
-    def _truncate(self):
-        """Truncate the timeserie."""
-        if self.max_size is not None:
-            # Remove empty points if any that could be added by aggregation
-            self.ts = self.ts[-self.max_size:]
+        return offset, serial.tobytes()
 
     @staticmethod
     def _resample_grouped(grouped_serie, agg_name, q=None):
