@@ -41,12 +41,13 @@ import six
 import sqlalchemy
 from sqlalchemy.engine import url as sqlalchemy_url
 import sqlalchemy.exc
-from sqlalchemy import types
+from sqlalchemy import types as sa_types
 import sqlalchemy_utils
 
 from gnocchi import exceptions
 from gnocchi import indexer
 from gnocchi.indexer import sqlalchemy_base as base
+from gnocchi.indexer import sqlalchemy_types as types
 from gnocchi import resource_type
 from gnocchi import utils
 
@@ -301,7 +302,7 @@ class SQLAlchemyIndexer(indexer.IndexerDriver):
         cfg = config.Config(
             "%s/alembic/alembic.ini" % os.path.dirname(__file__))
         cfg.set_main_option('sqlalchemy.url',
-                            self.conf.database.connection)
+                            self.conf.database.connection.replace('%', '%%'))
         return cfg
 
     def get_engine(self):
@@ -643,6 +644,10 @@ class SQLAlchemyIndexer(indexer.IndexerDriver):
         try:
             with self.facade.writer() as session:
                 session.add(apr)
+        except exception.DBReferenceError as e:
+            if e.constraint == 'fk_apr_ap_name_ap_name':
+                raise indexer.NoSuchArchivePolicy(archive_policy_name)
+            raise
         except exception.DBDuplicateEntry:
             raise indexer.ArchivePolicyRuleAlreadyExists(name)
         return apr
@@ -686,37 +691,60 @@ class SQLAlchemyIndexer(indexer.IndexerDriver):
         return m
 
     @retry_on_deadlock
-    def list_metrics(self, names=None, ids=None, details=False,
-                     status='active', limit=None, marker=None, sorts=None,
-                     creator=None, **kwargs):
+    def list_metrics(self, details=False, status='active',
+                     limit=None, marker=None, sorts=None,
+                     policy_filter=None, resource_policy_filter=None,
+                     attribute_filter=None):
         sorts = sorts or []
-        if ids is not None and not ids:
-            return []
-        if names is not None and not names:
-            return []
         with self.facade.independent_reader() as session:
             q = session.query(Metric).filter(
                 Metric.status == status)
-            if names is not None:
-                q = q.filter(Metric.name.in_(names))
-            if ids is not None:
-                q = q.filter(Metric.id.in_(ids))
-            if creator is not None:
-                if creator[0] == ":":
-                    q = q.filter(Metric.creator.like("%%%s" % creator))
-                elif creator[-1] == ":":
-                    q = q.filter(Metric.creator.like("%s%%" % creator))
-                else:
-                    q = q.filter(Metric.creator == creator)
-            for attr in kwargs:
-                q = q.filter(getattr(Metric, attr) == kwargs[attr])
             if details:
                 q = q.options(sqlalchemy.orm.joinedload('resource'))
+            if policy_filter or resource_policy_filter or attribute_filter:
+                engine = session.connection()
+                if attribute_filter:
+                    # We don't catch the indexer.QueryAttributeError error here
+                    # since we expect any user input on this function. If the
+                    # caller screws it, it's its problem: no need to convert
+                    # the exception to another type.
+                    attribute_f = QueryTransformer.build_filter(
+                        engine.dialect.name,
+                        Metric, attribute_filter)
+                    q = q.filter(attribute_f)
+                if policy_filter:
+                    # We don't catch the indexer.QueryAttributeError error here
+                    # since we expect any user input on this function. If the
+                    # caller screws it, it's its problem: no need to convert
+                    # the exception to another type.
+                    policy_f = QueryTransformer.build_filter(
+                        engine.dialect.name,
+                        Metric, policy_filter)
+                else:
+                    policy_f = None
+                if resource_policy_filter:
+                    q = q.join(Metric.resource)
+                    try:
+                        resource_policy_f = QueryTransformer.build_filter(
+                            engine.dialect.name,
+                            Resource,
+                            resource_policy_filter)
+                    except indexer.QueryAttributeError as e:
+                        # NOTE(jd) The QueryAttributeError does not know about
+                        # resource_type, so convert it
+                        raise indexer.ResourceAttributeError("generic",
+                                                             e.attribute)
+                else:
+                    resource_policy_f = None
+
+                if policy_filter or resource_policy_filter:
+                    q = q.filter(sqlalchemy.or_(policy_f, resource_policy_f))
 
             sort_keys, sort_dirs = self._build_sort_keys(sorts, ['id'])
 
             if marker:
-                metric_marker = self.list_metrics(ids=[marker])
+                metric_marker = self.list_metrics(
+                    attribute_filter={"in": {"id": [marker]}})
                 if metric_marker:
                     metric_marker = metric_marker[0]
                 else:
@@ -1115,7 +1143,7 @@ class SQLAlchemyIndexer(indexer.IndexerDriver):
         with self.facade.writer() as session:
             if session.query(Metric).filter(
                 Metric.id == id, Metric.status == 'active').update(
-                    {"status": "delete"}) == 0:
+                    {"status": "delete", "resource_id": None}) == 0:
                 raise indexer.NoSuchMetric(id)
 
     @staticmethod
@@ -1137,6 +1165,13 @@ class SQLAlchemyIndexer(indexer.IndexerDriver):
         return sort_keys, sort_dirs
 
 
+def _operator_in(field_name, value):
+    # Do not generate empty IN comparison
+    # https://github.com/gnocchixyz/gnocchi/issues/530
+    if len(value):
+        return field_name.in_(value)
+
+
 class QueryTransformer(object):
 
     unary_operators = {
@@ -1166,7 +1201,7 @@ class QueryTransformer(object):
         u"≠": operator.ne,
         u"ne": operator.ne,
 
-        u"in": lambda field_name, values: field_name.in_(values),
+        u"in": _operator_in,
 
         u"like": lambda field, value: field.like(value),
     }
@@ -1180,10 +1215,10 @@ class QueryTransformer(object):
     }
 
     converters = (
-        (base.TimestampUTC, utils.to_datetime),
-        (types.String, six.text_type),
-        (types.Integer, int),
-        (types.Numeric, float),
+        (types.TimestampUTC, utils.to_datetime),
+        (sa_types.String, six.text_type),
+        (sa_types.Integer, int),
+        (sa_types.Numeric, float),
     )
 
     @classmethod
@@ -1279,6 +1314,6 @@ class QueryTransformer(object):
                     op = cls.unary_operators[operator]
                 except KeyError:
                     raise indexer.QueryInvalidOperator(operator)
-                return cls._handle_unary_op(engine, op, nodes)
+                return cls._handle_unary_op(engine, table, op, nodes)
             return cls._handle_binary_op(engine, table, op, nodes)
         return cls._handle_multiple_op(engine, table, op, nodes)
