@@ -14,14 +14,13 @@
 import copy
 import itertools
 import operator
-import os
 import pkg_resources
 import uuid
 
 from oslo_config import cfg
-from oslo_middleware import cors
 
 import gnocchi.archive_policy
+import gnocchi.common.redis
 import gnocchi.indexer
 import gnocchi.storage
 import gnocchi.storage.ceph
@@ -41,6 +40,7 @@ class CustomStrSubWrapper(cfg.ConfigOpts.StrSubWrapper):
             return ''
         return value
 
+
 cfg.ConfigOpts.StrSubWrapper = CustomStrSubWrapper
 
 
@@ -48,6 +48,7 @@ _STORAGE_OPTS = list(itertools.chain(gnocchi.storage.OPTS,
                                      gnocchi.storage.ceph.OPTS,
                                      gnocchi.storage.file.OPTS,
                                      gnocchi.storage.swift.OPTS,
+                                     gnocchi.common.redis.OPTS,
                                      gnocchi.storage.s3.OPTS))
 
 
@@ -56,8 +57,82 @@ for opt in _INCOMING_OPTS:
     opt.default = '${storage.%s}' % opt.name
 
 
+API_OPTS = (
+    cfg.HostAddressOpt('host',
+                       default="0.0.0.0",
+                       help="Host to listen on"),
+    cfg.PortOpt('port',
+                default=8041,
+                help="Port to listen on"),
+    cfg.StrOpt('uwsgi-mode',
+               default='http',
+               choices=["http", "http-socket", "socket"],
+               help="""Socket type to use for uWSGI:
+* http: support HTTP/1.1 and keepalive,
+  but not chunked encoding (InfluxDB)
+* http-socket/socket: support chunked encoding, but require a upstream HTTP
+  Server for HTTP/1.1, keepalive and HTTP protocol correctness.
+""")
+)
+
+
+_cli_options = (
+    cfg.BoolOpt(
+        'debug',
+        short='d',
+        default=False,
+        help='If set to true, the logging level will be set to DEBUG.'),
+    cfg.BoolOpt(
+        'verbose',
+        short='v',
+        default=True,
+        help='If set to true, the logging level will be set to INFO.'),
+    cfg.StrOpt(
+        "log-dir",
+        help="Base directory for log files. "
+        "If not set, logging will go to stderr."),
+    cfg.StrOpt(
+        'log-file',
+        metavar='PATH',
+        help='(Optional) Name of log file to send logging output to. '
+        'If no default is set, logging will go to stderr as '
+        'defined by use_stderr.'),
+)
+
+
 def list_opts():
     return [
+        ("DEFAULT", _cli_options + (
+            cfg.StrOpt(
+                'coordination_url',
+                secret=True,
+                deprecated_group="storage",
+                help='Coordination driver URL'),
+            cfg.IntOpt(
+                'parallel_operations',
+                min=1,
+                deprecated_name='aggregation_workers_number',
+                deprecated_group='storage',
+                help='Number of threads to use to parallelize '
+                'some operations. '
+                'Default is set to the number of CPU available.'),
+            cfg.BoolOpt(
+                'use-syslog',
+                default=False,
+                help='Use syslog for logging.'),
+            cfg.BoolOpt(
+                'use-journal',
+                default=False,
+                help='Enable journald for logging. '
+                'If running in a systemd environment you may wish '
+                'to enable journal support. Doing so will use the '
+                'journal native protocol which includes structured '
+                'metadata in addition to log messages.'),
+            cfg.StrOpt(
+                'syslog-log-facility',
+                default='user',
+                help='Syslog facility to receive log lines.')
+        )),
         ("indexer", gnocchi.indexer.OPTS),
         ("metricd", (
             cfg.IntOpt('workers', min=1,
@@ -65,11 +140,16 @@ def list_opts():
                        help='Number of workers for Gnocchi metric daemons. '
                        'By default the available number of CPU is used.'),
             cfg.IntOpt('metric_processing_delay',
-                       default=30,
+                       default=60,
                        required=True,
                        deprecated_group='storage',
                        help="How many seconds to wait between "
                        "scheduling new metrics to process"),
+            cfg.BoolOpt(
+                'greedy', default=True,
+                help="Allow to bypass `metric_processing_delay` if metricd "
+                "is notified that measures are ready to be processed."
+            ),
             cfg.IntOpt('metric_reporting_delay',
                        deprecated_group='storage',
                        default=120,
@@ -84,18 +164,17 @@ def list_opts():
                        required=True,
                        help="How many seconds to wait between "
                        "cleaning of expired data"),
-            cfg.IntOpt('tasks_per_worker',
-                       default=256,
+            cfg.IntOpt('processing_replicas',
+                       default=3,
                        min=1,
-                       help="How many tasks to assign each metricd worker "
-                       "when scheduling measures processing jobs"),
+                       help="Number of workers that share a task. A higher "
+                       "value may improve worker utilization but may also "
+                       "increase load on coordination backend. Value is "
+                       "capped by number of workers globally."),
         )),
         ("api", (
             cfg.StrOpt('paste_config',
-                       default=os.path.abspath(
-                           os.path.join(
-                               os.path.dirname(__file__),
-                               "rest", "api-paste.ini")),
+                       default="api-paste.ini",
                        help='Path to API Paste configuration.'),
             cfg.StrOpt('auth_mode',
                        default="basic",
@@ -108,13 +187,19 @@ def list_opts():
                        required=True,
                        help=('The maximum number of items returned in a '
                              'single response from a collection resource')),
-        )),
-        ("storage", (_STORAGE_OPTS + gnocchi.storage._carbonara.OPTS)),
+            cfg.IntOpt('operation_timeout',
+                       deprecated_name="refresh_timeout",
+                       default=10, min=0,
+                       help='Number of seconds before timeout when attempting '
+                            'to do some operations.'),
+        ) + API_OPTS,
+        ),
+        ("storage", _STORAGE_OPTS),
         ("incoming", _INCOMING_OPTS),
         ("statsd", (
-            cfg.StrOpt('host',
-                       default='0.0.0.0',
-                       help='The listen IP for statsd'),
+            cfg.HostAddressOpt('host',
+                               default='0.0.0.0',
+                               help='The listen IP for statsd'),
             cfg.PortOpt('port',
                         default=8125,
                         help='The port for statsd'),
@@ -122,17 +207,8 @@ def list_opts():
                 'resource_id',
                 type=uuid.UUID,
                 help='Resource UUID to use to identify statsd in Gnocchi'),
-            cfg.StrOpt(
-                'user_id',
-                deprecated_for_removal=True,
-                help='User ID to use to identify statsd in Gnocchi'),
-            cfg.StrOpt(
-                'project_id',
-                deprecated_for_removal=True,
-                help='Project ID to use to identify statsd in Gnocchi'),
             cfg.StrOpt(
                 'creator',
-                default="${statsd.user_id}:${statsd.project_id}",
                 help='Creator value to use to identify statsd in Gnocchi'),
             cfg.StrOpt(
                 'archive_policy_name',
@@ -147,8 +223,10 @@ def list_opts():
 
 
 def set_defaults():
+    from oslo_middleware import cors
     cfg.set_defaults(cors.CORS_OPTS,
                      allow_headers=[
+                         'Authorization',
                          'X-Auth-Token',
                          'X-Subject-Token',
                          'X-User-Id',
