@@ -14,7 +14,6 @@
 # WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 # License for the specific language governing permissions and limitations
 # under the License.
-import collections
 import functools
 import itertools
 import operator
@@ -39,6 +38,9 @@ OPTS = [
 LOG = daiquiri.getLogger(__name__)
 
 
+ITEMGETTER_1 = operator.itemgetter(1)
+
+
 class StorageError(Exception):
     pass
 
@@ -59,23 +61,25 @@ class MetricDoesNotExist(StorageError):
 class AggregationDoesNotExist(StorageError):
     """Error raised when the aggregation method doesn't exists for a metric."""
 
-    def __init__(self, metric, method):
+    def __init__(self, metric, method, granularity):
         self.metric = metric
         self.method = method
-        super(AggregationDoesNotExist, self).__init__(
-            "Aggregation method '%s' for metric %s does not exist" %
-            (method, metric))
-
-
-class GranularityDoesNotExist(StorageError):
-    """Error raised when the granularity doesn't exist for a metric."""
-
-    def __init__(self, metric, granularity):
-        self.metric = metric
         self.granularity = granularity
-        super(GranularityDoesNotExist, self).__init__(
-            "Granularity '%s' for metric %s does not exist" %
-            (utils.timespan_total_seconds(granularity), metric))
+        super(AggregationDoesNotExist, self).__init__(
+            "Aggregation method '%s' at granularity '%s' "
+            "for metric %s does not exist" %
+            (method, utils.timespan_total_seconds(granularity), metric))
+
+    def jsonify(self):
+        return {
+            "cause": "Aggregation does not exist",
+            "detail": {
+                # FIXME(jd) Pecan does not use our JSON renderer for errors
+                # So we need to convert this
+                "granularity": utils.timespan_total_seconds(self.granularity),
+                "aggregation_method": self.method,
+            },
+        }
 
 
 class MetricAlreadyExists(StorageError):
@@ -106,6 +110,7 @@ class SackLockTimeoutError(StorageError):
         pass
 
 
+@utils.retry_on_exception_and_log("Unable to initialize storage driver")
 def get_driver(conf, coord):
     """Return the configured driver."""
     return utils.get_driver_class('gnocchi.storage', conf.storage)(
@@ -121,8 +126,14 @@ class StorageDriver(object):
     def upgrade():
         pass
 
+    def _get_measures(self, metric, keys, aggregation, version=3):
+        return utils.parallel_map(
+            self._get_measures_unbatched,
+            ((metric, key, aggregation, version)
+             for key in keys))
+
     @staticmethod
-    def _get_measures(metric, timestamp_key, aggregation, version=3):
+    def _get_measures_unbatched(metric, timestamp_key, aggregation, version=3):
         raise NotImplementedError
 
     @staticmethod
@@ -186,111 +197,110 @@ class StorageDriver(object):
         """
         return name.split("_")[-1] == 'v%s' % v
 
-    def get_measures(self, metric, from_timestamp=None, to_timestamp=None,
-                     aggregation='mean', granularity=None, resample=None):
+    def get_measures(self, metric, granularities,
+                     from_timestamp=None, to_timestamp=None,
+                     aggregation='mean', resample=None):
         """Get a measure to a metric.
 
         :param metric: The metric measured.
+        :param granularities: The granularities to retrieve.
         :param from timestamp: The timestamp to get the measure from.
         :param to timestamp: The timestamp to get the measure to.
         :param aggregation: The type of aggregation to retrieve.
-        :param granularity: The granularity to retrieve.
         :param resample: The granularity to resample to.
         """
-        if aggregation not in metric.archive_policy.aggregation_methods:
-            raise AggregationDoesNotExist(metric, aggregation)
 
-        if granularity is None:
-            agg_timeseries = utils.parallel_map(
-                self._get_measures_timeserie,
-                ((metric, aggregation, ap.granularity,
-                  from_timestamp, to_timestamp)
-                 for ap in reversed(metric.archive_policy.definition)))
-        else:
-            agg_timeseries = [self._get_measures_timeserie(
-                metric, aggregation, granularity,
-                from_timestamp, to_timestamp)]
+        aggregations = []
+        for g in sorted(granularities, reverse=True):
+            agg = metric.archive_policy.get_aggregation(aggregation, g)
+            if agg is None:
+                raise AggregationDoesNotExist(metric, aggregation, g)
+            aggregations.append(agg)
 
-        if resample and granularity:
+        agg_timeseries = utils.parallel_map(
+            self._get_measures_timeserie,
+            ((metric, ag, from_timestamp, to_timestamp)
+             for ag in aggregations))
+
+        if resample:
             agg_timeseries = list(map(lambda agg: agg.resample(resample),
                                       agg_timeseries))
 
         return list(itertools.chain(*[ts.fetch(from_timestamp, to_timestamp)
                                       for ts in agg_timeseries]))
 
-    def _get_measures_and_unserialize(self, metric, key, aggregation):
-        data = self._get_measures(metric, key, aggregation)
-        try:
-            return carbonara.AggregatedTimeSerie.unserialize(
-                data, key, aggregation)
-        except carbonara.InvalidData:
-            LOG.error("Data corruption detected for %s "
-                      "aggregated `%s' timeserie, granularity `%s' "
-                      "around time `%s', ignoring.",
-                      metric.id, aggregation, key.sampling, key)
+    def _get_measures_and_unserialize(self, metric, keys, aggregation):
+        if not keys:
+            return []
+        raw_measures = self._get_measures(metric, keys, aggregation)
+        results = []
+        for key, raw in six.moves.zip(keys, raw_measures):
+            try:
+                results.append(carbonara.AggregatedTimeSerie.unserialize(
+                    raw, key, aggregation))
+            except carbonara.InvalidData:
+                LOG.error("Data corruption detected for %s "
+                          "aggregated `%s' timeserie, granularity `%s' "
+                          "around time `%s', ignoring.",
+                          metric.id, aggregation, key.sampling, key)
+        return results
 
-    def _get_measures_timeserie(self, metric,
-                                aggregation, granularity,
+    def _get_measures_timeserie(self, metric, aggregation,
                                 from_timestamp=None, to_timestamp=None):
-
-        # Find the number of point
-        for d in metric.archive_policy.definition:
-            if d.granularity == granularity:
-                points = d.points
-                break
-        else:
-            raise GranularityDoesNotExist(metric, granularity)
-
-        all_keys = None
         try:
             all_keys = self._list_split_keys_for_metric(
-                metric, aggregation, granularity)
+                metric, aggregation.method, aggregation.granularity)
         except MetricDoesNotExist:
-            for d in metric.archive_policy.definition:
-                if d.granularity == granularity:
-                    return carbonara.AggregatedTimeSerie(
-                        sampling=granularity,
-                        aggregation_method=aggregation,
-                        max_size=d.points)
-            raise GranularityDoesNotExist(metric, granularity)
+            return carbonara.AggregatedTimeSerie(
+                sampling=aggregation.granularity,
+                aggregation_method=aggregation.method)
 
         if from_timestamp:
             from_timestamp = carbonara.SplitKey.from_timestamp_and_sampling(
-                from_timestamp, granularity)
+                from_timestamp, aggregation.granularity)
 
         if to_timestamp:
             to_timestamp = carbonara.SplitKey.from_timestamp_and_sampling(
-                to_timestamp, granularity)
+                to_timestamp, aggregation.granularity)
 
-        timeseries = list(filter(
-            lambda x: x is not None,
-            utils.parallel_map(
-                self._get_measures_and_unserialize,
-                ((metric, key, aggregation)
-                 for key in sorted(all_keys)
-                 if ((not from_timestamp or key >= from_timestamp)
-                     and (not to_timestamp or key <= to_timestamp))))
-        ))
+        keys = [key for key in sorted(all_keys)
+                if ((not from_timestamp or key >= from_timestamp)
+                    and (not to_timestamp or key <= to_timestamp))]
 
-        return carbonara.AggregatedTimeSerie.from_timeseries(
-            sampling=granularity,
-            aggregation_method=aggregation,
-            timeseries=timeseries,
-            max_size=points)
+        timeseries = self._get_measures_and_unserialize(
+            metric, keys, aggregation.method)
+
+        ts = carbonara.AggregatedTimeSerie.from_timeseries(
+            sampling=aggregation.granularity,
+            aggregation_method=aggregation.method,
+            timeseries=timeseries)
+        # We need to truncate because:
+        # - If the driver is not in WRITE_FULL mode, then it might read too
+        # much data that will be deleted once the split is rewritten. Just
+        # truncate so we don't return it.
+        # - If the driver is in WRITE_FULL but the archive policy has been
+        # resized, we might still have too much points stored, which will be
+        # deleted at a later point when new points will be procecessed.
+        # Truncate to be sure we don't return them.
+        if aggregation.timespan is not None:
+            ts.truncate(aggregation.timespan)
+        return ts
 
     def _store_timeserie_split(self, metric, key, split,
-                               aggregation, oldest_mutable_timestamp):
+                               aggregation, oldest_mutable_timestamp,
+                               oldest_point_to_keep):
         # NOTE(jd) We write the full split only if the driver works that way
         # (self.WRITE_FULL) or if the oldest_mutable_timestamp is out of range.
         write_full = self.WRITE_FULL or next(key) <= oldest_mutable_timestamp
         if write_full:
             try:
                 existing = self._get_measures_and_unserialize(
-                    metric, key, aggregation)
+                    metric, [key], aggregation)
             except AggregationDoesNotExist:
                 pass
             else:
-                if existing is not None:
+                if existing:
+                    existing = existing[0]
                     if split is not None:
                         existing.merge(split)
                     split = existing
@@ -309,13 +319,15 @@ class StorageDriver(object):
                         aggregation, key)
             return
 
+        if oldest_point_to_keep is not None:
+            split.truncate(oldest_point_to_keep)
+
         offset, data = split.serialize(key, compressed=write_full)
 
         return self._store_metric_measures(metric, key, aggregation,
                                            data, offset=offset)
 
-    def _add_measures(self, aggregation, archive_policy_def,
-                      metric, grouped_serie,
+    def _add_measures(self, aggregation, ap_def, metric, grouped_serie,
                       previous_oldest_mutable_timestamp,
                       oldest_mutable_timestamp):
 
@@ -326,8 +338,7 @@ class StorageDriver(object):
             aggregation_to_compute = aggregation
 
         ts = carbonara.AggregatedTimeSerie.from_grouped_serie(
-            grouped_serie, archive_policy_def.granularity,
-            aggregation_to_compute, max_size=archive_policy_def.points)
+            grouped_serie, ap_def.granularity, aggregation_to_compute)
 
         # Don't do anything if the timeserie is empty
         if not ts:
@@ -340,45 +351,57 @@ class StorageDriver(object):
             and previous_oldest_mutable_timestamp is not None
         )
 
-        if archive_policy_def.timespan or need_rewrite:
-            existing_keys = self._list_split_keys_for_metric(
-                metric, aggregation, archive_policy_def.granularity)
-
-        # First delete old splits
-        if archive_policy_def.timespan:
-            oldest_point_to_keep = ts.last - archive_policy_def.timespan
+        if ap_def.timespan:
+            oldest_point_to_keep = ts.last - ap_def.timespan
             oldest_key_to_keep = ts.get_split_key(oldest_point_to_keep)
-            for key in list(existing_keys):
-                # NOTE(jd) Only delete if the key is strictly inferior to
-                # the timestamp; we don't delete any timeserie split that
-                # contains our timestamp, so we prefer to keep a bit more
-                # than deleting too much
-                if key < oldest_key_to_keep:
-                    self._delete_metric_measures(metric, key, aggregation)
-                    existing_keys.remove(key)
         else:
+            oldest_point_to_keep = None
             oldest_key_to_keep = None
 
-        # Rewrite all read-only splits just for fun (and compression). This
-        # only happens if `previous_oldest_mutable_timestamp' exists, which
-        # means we already wrote some splits at some point – so this is not the
-        # first time we treat this timeserie.
-        if need_rewrite:
+        if previous_oldest_mutable_timestamp and (ap_def.timespan or
+                                                  need_rewrite):
             previous_oldest_mutable_key = ts.get_split_key(
                 previous_oldest_mutable_timestamp)
             oldest_mutable_key = ts.get_split_key(oldest_mutable_timestamp)
 
+            # only cleanup if there is a new object, as there must be a new
+            # object for an old object to be cleanup
             if previous_oldest_mutable_key != oldest_mutable_key:
-                for key in existing_keys:
-                    if previous_oldest_mutable_key <= key < oldest_mutable_key:
-                        LOG.debug(
-                            "Compressing previous split %s (%s) for metric %s",
-                            key, aggregation, metric)
-                        # NOTE(jd) Rewrite it entirely for fun (and later for
-                        # compression). For that, we just pass None as split.
-                        self._store_timeserie_split(
-                            metric, key,
-                            None, aggregation, oldest_mutable_timestamp)
+                existing_keys = sorted(self._list_split_keys_for_metric(
+                    metric, aggregation, ap_def.granularity))
+
+                # First, check for old splits to delete
+                if ap_def.timespan:
+                    oldest_point_to_keep = ts.last - ap_def.timespan
+                    oldest_key_to_keep = ts.get_split_key(oldest_point_to_keep)
+                    for key in list(existing_keys):
+                        # NOTE(jd) Only delete if the key is strictly inferior
+                        # the timestamp; we don't delete any timeserie split
+                        # that contains our timestamp, so we prefer to keep a
+                        # bit more than deleting too much
+                        if key >= oldest_key_to_keep:
+                            break
+                        self._delete_metric_measures(metric, key, aggregation)
+                        existing_keys.remove(key)
+
+                # Rewrite all read-only splits just for fun (and compression).
+                # This only happens if `previous_oldest_mutable_timestamp'
+                # exists, which means we already wrote some splits at some
+                # point – so this is not the first time we treat this
+                # timeserie.
+                if need_rewrite:
+                    for key in existing_keys:
+                        if previous_oldest_mutable_key <= key:
+                            if key >= oldest_mutable_key:
+                                break
+                            LOG.debug("Compressing previous split %s (%s) for "
+                                      "metric %s", key, aggregation, metric)
+                            # NOTE(jd) Rewrite it entirely for fun (and later
+                            # for compression). For that, we just pass None as
+                            # split.
+                            self._store_timeserie_split(
+                                metric, key, None, aggregation,
+                                oldest_mutable_timestamp, oldest_point_to_keep)
 
         for key, split in ts.split():
             if oldest_key_to_keep is None or key >= oldest_key_to_keep:
@@ -386,26 +409,13 @@ class StorageDriver(object):
                     "Storing split %s (%s) for metric %s",
                     key, aggregation, metric)
                 self._store_timeserie_split(
-                    metric, key, split, aggregation, oldest_mutable_timestamp)
+                    metric, key, split, aggregation, oldest_mutable_timestamp,
+                    oldest_point_to_keep)
 
     @staticmethod
     def _delete_metric(metric):
         raise NotImplementedError
 
-    def delete_metric(self, incoming, metric, sync=False):
-        LOG.debug("Deleting metric %s", metric)
-        lock = incoming.get_sack_lock(
-            self.coord, incoming.sack_for_metric(metric.id))
-        if not lock.acquire(blocking=sync):
-            raise LockedMetric(metric)
-        # NOTE(gordc): no need to hold lock because the metric has been already
-        #              marked as "deleted" in the indexer so no measure worker
-        #              is going to process it anymore.
-        lock.release()
-        self._delete_metric(metric)
-        incoming.delete_unprocessed_measures_for_metric_id(metric.id)
-        LOG.debug("Deleted metric %s", metric)
-
     @staticmethod
     def _delete_metric_measures(metric, timestamp_key,
                                 aggregation, granularity, version=3):
@@ -433,21 +443,47 @@ class StorageDriver(object):
                      on error
         :type sync: bool
         """
-
-        metrics_to_expunge = index.list_metrics(status='delete')
-        for m in metrics_to_expunge:
+        # FIXME(jd) The indexer could return them sorted/grouped by directly
+        metrics_to_expunge = sorted(
+            ((m, incoming.sack_for_metric(m.id))
+             for m in index.list_metrics(status='delete')),
+            key=ITEMGETTER_1)
+        for sack, metrics in itertools.groupby(
+                metrics_to_expunge, key=ITEMGETTER_1):
             try:
-                self.delete_metric(incoming, m, sync)
-                index.expunge_metric(m.id)
-            except (indexer.NoSuchMetric, LockedMetric):
-                # It's possible another process deleted or is deleting the
-                # metric, not a big deal
-                pass
+                lock = incoming.get_sack_lock(self.coord, sack)
+                if not lock.acquire(blocking=sync):
+                    # Retry later
+                    LOG.debug(
+                        "Sack %s is locked, cannot expunge metrics", sack)
+                    continue
+                # NOTE(gordc): no need to hold lock because the metric has been
+                # already marked as "deleted" in the indexer so no measure
+                # worker is going to process it anymore.
+                lock.release()
             except Exception:
                 if sync:
                     raise
-                LOG.error("Unable to expunge metric %s from storage", m,
-                          exc_info=True)
+                LOG.error("Unable to lock sack %s for expunging metrics",
+                          sack, exc_info=True)
+            else:
+                for metric, sack in metrics:
+                    LOG.debug("Deleting metric %s", metric)
+                    try:
+                        incoming.delete_unprocessed_measures_for_metric(
+                            metric.id)
+                        self._delete_metric(metric)
+                        try:
+                            index.expunge_metric(metric.id)
+                        except indexer.NoSuchMetric:
+                            # It's possible another process deleted or is
+                            # deleting the metric, not a big deal
+                            pass
+                    except Exception:
+                        if sync:
+                            raise
+                        LOG.error("Unable to expunge metric %s from storage",
+                                  metric, exc_info=True)
 
     def process_new_measures(self, indexer, incoming, metrics_to_process,
                              sync=False):
@@ -458,12 +494,13 @@ class StorageDriver(object):
         """
         # process only active metrics. deleted metrics with unprocessed
         # measures will be skipped until cleaned by janitor.
-        metrics = indexer.list_metrics(ids=metrics_to_process)
+        metrics = indexer.list_metrics(
+            attribute_filter={"in": {"id": metrics_to_process}})
         for metric in metrics:
             # NOTE(gordc): must lock at sack level
             try:
                 LOG.debug("Processing measures for %s", metric)
-                with incoming.process_measure_for_metric(metric) \
+                with incoming.process_measure_for_metric(metric.id) \
                         as measures:
                     self._compute_and_store_timeseries(metric, measures)
                 LOG.debug("Measures for metric %s processed", metric)
@@ -479,7 +516,7 @@ class StorageDriver(object):
             LOG.debug("Skipping %s (already processed)", metric)
             return
 
-        measures.sort(order='timestamps')
+        measures = numpy.sort(measures, order='timestamps')
 
         agg_methods = list(metric.archive_policy.aggregation_methods)
         block_size = metric.archive_policy.max_block_size
@@ -557,53 +594,18 @@ class StorageDriver(object):
 
         self._store_unaggregated_timeserie(metric, ts.serialize())
 
-    def _find_measure(self, metric, aggregation, granularity, predicate,
-                      from_timestamp, to_timestamp):
+    def find_measure(self, metric, predicate, granularity, aggregation="mean",
+                     from_timestamp=None, to_timestamp=None):
+        agg = metric.archive_policy.get_aggregation(aggregation, granularity)
+        if agg is None:
+            raise AggregationDoesNotExist(metric, aggregation, granularity)
+
         timeserie = self._get_measures_timeserie(
-            metric, aggregation, granularity,
-            from_timestamp, to_timestamp)
+            metric, agg, from_timestamp, to_timestamp)
         values = timeserie.fetch(from_timestamp, to_timestamp)
-        return {metric:
-                [(timestamp, g, value)
-                 for timestamp, g, value in values
-                 if predicate(value)]}
-
-    def search_value(self, metrics, query, from_timestamp=None,
-                     to_timestamp=None, aggregation='mean',
-                     granularity=None):
-        """Search for an aggregated value that realizes a predicate.
-
-        :param metrics: The list of metrics to look into.
-        :param query: The query being sent.
-        :param from_timestamp: The timestamp to get the measure from.
-        :param to_timestamp: The timestamp to get the measure to.
-        :param aggregation: The type of aggregation to retrieve.
-        :param granularity: The granularity to retrieve.
-        """
-
-        granularity = granularity or []
-        predicate = MeasureQuery(query)
-
-        results = utils.parallel_map(
-            self._find_measure,
-            [(metric, aggregation,
-              gran, predicate,
-              from_timestamp, to_timestamp)
-             for metric in metrics
-             for gran in granularity or
-             (defin.granularity
-              for defin in metric.archive_policy.definition)])
-        result = collections.defaultdict(list)
-        for r in results:
-            for metric, metric_result in six.iteritems(r):
-                result[metric].extend(metric_result)
-
-        # Sort the result
-        for metric, r in six.iteritems(result):
-            # Sort by timestamp asc, granularity desc
-            r.sort(key=lambda t: (t[0], - t[1]))
-
-        return result
+        return [(timestamp, g, value)
+                for timestamp, g, value in values
+                if predicate(value)]
 
 
 class MeasureQuery(object):
