@@ -13,18 +13,14 @@
 # WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 # License for the specific language governing permissions and limitations
 # under the License.
-import errno
 
 from oslo_config import cfg
-from oslo_log import log
 
+from gnocchi.common import ceph
 from gnocchi import storage
-from gnocchi.storage import _carbonara
-from gnocchi.storage.common import ceph
+from gnocchi import utils
 
 
-LOG = log.getLogger(__name__)
-
 OPTS = [
     cfg.StrOpt('ceph_pool',
                default='gnocchi',
@@ -44,51 +40,27 @@ OPTS = [
 rados = ceph.rados
 
 
-class CephStorage(_carbonara.CarbonaraBasedStorage):
+class CephStorage(storage.StorageDriver):
     WRITE_FULL = False
 
-    def __init__(self, conf, incoming):
-        super(CephStorage, self).__init__(conf, incoming)
+    def __init__(self, conf, coord=None):
+        super(CephStorage, self).__init__(conf, coord)
         self.rados, self.ioctx = ceph.create_rados_connection(conf)
 
+    def __str__(self):
+        # Use cluster ID for now
+        return "%s: %s" % (self.__class__.__name__, self.rados.get_fsid())
+
     def stop(self):
         ceph.close_rados_connection(self.rados, self.ioctx)
         super(CephStorage, self).stop()
 
-    def _check_for_metric_upgrade(self, metric):
-        lock = self._lock(metric.id)
-        with lock:
-            container = "gnocchi_%s_container" % metric.id
-            unagg_obj = self._build_unaggregated_timeserie_path(metric, 3)
-            try:
-                xattrs = tuple(k for k, v in self.ioctx.get_xattrs(container))
-            except rados.ObjectNotFound:
-                # this means already upgraded or some corruption? move on.
-                pass
-            else:
-                # if xattrs are found, it means we're coming from
-                # gnocchiv2. migrate to omap accordingly.
-                if xattrs:
-                    keys = xattrs
-                # if no xattrs but object exists, it means it already
-                # migrated to v3 and now upgrade to use single object
-                else:
-                    with rados.ReadOpCtx() as op:
-                        omaps, ret = self.ioctx.get_omap_vals(op, "", "", -1)
-                        self.ioctx.operate_read_op(op, container)
-                        keys = (k for k, __ in omaps)
-                with rados.WriteOpCtx() as op:
-                    self.ioctx.set_omap(op, keys,
-                                        tuple([b""] * len(keys)))
-                    self.ioctx.operate_write_op(op, unagg_obj)
-                self.ioctx.remove_object(container)
-        super(CephStorage, self)._check_for_metric_upgrade(metric)
-
     @staticmethod
-    def _get_object_name(metric, timestamp_key, aggregation, granularity,
-                         version=3):
+    def _get_object_name(metric, key, aggregation, version=3):
         name = str("gnocchi_%s_%s_%s_%s" % (
-            metric.id, timestamp_key, aggregation, granularity))
+            metric.id, key, aggregation,
+            utils.timespan_total_seconds(key.sampling)),
+        )
         return name + '_v%s' % version if version else name
 
     def _object_exists(self, name):
@@ -105,10 +77,9 @@ class CephStorage(_carbonara.CarbonaraBasedStorage):
         else:
             self.ioctx.write_full(name, b"")
 
-    def _store_metric_measures(self, metric, timestamp_key, aggregation,
-                               granularity, data, offset=None, version=3):
-        name = self._get_object_name(metric, timestamp_key,
-                                     aggregation, granularity, version)
+    def _store_metric_measures(self, metric, key, aggregation,
+                               data, offset=None, version=3):
+        name = self._get_object_name(metric, key, aggregation, version)
         if offset is None:
             self.ioctx.write_full(name, data)
         else:
@@ -118,15 +89,21 @@ class CephStorage(_carbonara.CarbonaraBasedStorage):
             self.ioctx.operate_write_op(
                 op, self._build_unaggregated_timeserie_path(metric, 3))
 
-    def _delete_metric_measures(self, metric, timestamp_key, aggregation,
-                                granularity, version=3):
-        name = self._get_object_name(metric, timestamp_key,
-                                     aggregation, granularity, version)
+    def _delete_metric_measures(self, metric, key, aggregation, version=3):
+        name = self._get_object_name(metric, key, aggregation, version)
+
+        try:
+            self.ioctx.remove_object(name)
+        except rados.ObjectNotFound:
+            # It's possible that we already remove that object and then crashed
+            # before removing it from the OMAP key list; then no big deal
+            # anyway.
+            pass
+
         with rados.WriteOpCtx() as op:
             self.ioctx.remove_omap_keys(op, (name,))
             self.ioctx.operate_write_op(
                 op, self._build_unaggregated_timeserie_path(metric, 3))
-        self.ioctx.aio_remove(name)
 
     def _delete_metric(self, metric):
         with rados.ReadOpCtx() as op:
@@ -136,28 +113,43 @@ class CephStorage(_carbonara.CarbonaraBasedStorage):
                     op, self._build_unaggregated_timeserie_path(metric, 3))
             except rados.ObjectNotFound:
                 return
-            if ret == errno.ENOENT:
-                return
-            for name, _ in omaps:
-                self.ioctx.aio_remove(name)
-        self.ioctx.aio_remove(
-            self._build_unaggregated_timeserie_path(metric, 3))
 
-    def _get_measures(self, metric, timestamp_key, aggregation, granularity,
-                      version=3):
+            # NOTE(sileht): after reading the libradospy, I'm
+            # not sure that ret will have the correct value
+            # get_omap_vals transforms the C int to python int
+            # before operate_read_op is called, I dunno if the int
+            # content is copied during this transformation or if
+            # this is a pointer to the C int, I think it's copied...
+            try:
+                ceph.errno_to_exception(ret)
+            except rados.ObjectNotFound:
+                return
+
+        ops = [self.ioctx.aio_remove(name) for name, _ in omaps]
+
+        for op in ops:
+            op.wait_for_complete_and_cb()
+
         try:
-            name = self._get_object_name(metric, timestamp_key,
-                                         aggregation, granularity, version)
+            self.ioctx.remove_object(
+                self._build_unaggregated_timeserie_path(metric, 3))
+        except rados.ObjectNotFound:
+            # It's possible that the object does not exists
+            pass
+
+    def _get_measures_unbatched(self, metric, key, aggregation, version=3):
+        try:
+            name = self._get_object_name(metric, key, aggregation, version)
             return self._get_object_content(name)
         except rados.ObjectNotFound:
             if self._object_exists(
                     self._build_unaggregated_timeserie_path(metric, 3)):
-                raise storage.AggregationDoesNotExist(metric, aggregation)
+                raise storage.AggregationDoesNotExist(
+                    metric, aggregation, key.sampling)
             else:
                 raise storage.MetricDoesNotExist(metric)
 
-    def _list_split_keys_for_metric(self, metric, aggregation, granularity,
-                                    version=None):
+    def _list_split_keys(self, metric, aggregation, granularity, version=3):
         with rados.ReadOpCtx() as op:
             omaps, ret = self.ioctx.get_omap_vals(op, "", "", -1)
             try:
@@ -165,12 +157,23 @@ class CephStorage(_carbonara.CarbonaraBasedStorage):
                     op, self._build_unaggregated_timeserie_path(metric, 3))
             except rados.ObjectNotFound:
                 raise storage.MetricDoesNotExist(metric)
-            if ret == errno.ENOENT:
+
+            # NOTE(sileht): after reading the libradospy, I'm
+            # not sure that ret will have the correct value
+            # get_omap_vals transforms the C int to python int
+            # before operate_read_op is called, I dunno if the int
+            # content is copied during this transformation or if
+            # this is a pointer to the C int, I think it's copied...
+            try:
+                ceph.errno_to_exception(ret)
+            except rados.ObjectNotFound:
                 raise storage.MetricDoesNotExist(metric)
+
             keys = set()
+            granularity = str(utils.timespan_total_seconds(granularity))
             for name, value in omaps:
                 meta = name.split('_')
-                if (aggregation == meta[3] and granularity == float(meta[4])
+                if (aggregation == meta[3] and granularity == meta[4]
                         and self._version_check(name, version)):
                     keys.add(meta[2])
             return keys
@@ -191,10 +194,6 @@ class CephStorage(_carbonara.CarbonaraBasedStorage):
         self.ioctx.write_full(
             self._build_unaggregated_timeserie_path(metric, version), data)
 
-    def _delete_unaggregated_timeserie(self, metric, version=3):
-        self.ioctx.aio_remove(
-            self._build_unaggregated_timeserie_path(metric, version))
-
     def _get_object_content(self, name):
         offset = 0
         content = b''
