@@ -22,7 +22,6 @@ import tempfile
 from oslo_config import cfg
 
 from gnocchi import storage
-from gnocchi.storage import _carbonara
 from gnocchi import utils
 
 
@@ -33,15 +32,20 @@ OPTS = [
 ]
 
 
-class FileStorage(_carbonara.CarbonaraBasedStorage):
+class FileStorage(storage.StorageDriver):
     WRITE_FULL = True
 
-    def __init__(self, conf, incoming):
-        super(FileStorage, self).__init__(conf, incoming)
+    def __init__(self, conf, coord=None):
+        super(FileStorage, self).__init__(conf, coord)
         self.basepath = conf.file_basepath
         self.basepath_tmp = os.path.join(self.basepath, 'tmp')
+
+    def upgrade(self):
         utils.ensure_paths([self.basepath_tmp])
 
+    def __str__(self):
+        return "%s: %s" % (self.__class__.__name__, str(self.basepath))
+
     def _atomic_file_store(self, dest, data):
         tmpfile = tempfile.NamedTemporaryFile(
             prefix='gnocchi', dir=self.basepath_tmp,
@@ -63,9 +67,12 @@ class FileStorage(_carbonara.CarbonaraBasedStorage):
                             "agg_" + aggregation)
 
     def _build_metric_path_for_split(self, metric, aggregation,
-                                     timestamp_key, granularity, version=3):
-        path = os.path.join(self._build_metric_path(metric, aggregation),
-                            timestamp_key + "_" + str(granularity))
+                                     key, version=3):
+        path = os.path.join(
+            self._build_metric_path(metric, aggregation),
+            str(key)
+            + "_"
+            + str(utils.timespan_total_seconds(key.sampling)))
         return path + '_v%s' % version if version else path
 
     def _create_metric(self, metric):
@@ -84,9 +91,9 @@ class FileStorage(_carbonara.CarbonaraBasedStorage):
                     raise
 
     def _store_unaggregated_timeserie(self, metric, data, version=3):
-        self._atomic_file_store(
-            self._build_unaggregated_timeserie_path(metric, version),
-            data)
+        dest = self._build_unaggregated_timeserie_path(metric, version)
+        with open(dest, "wb") as f:
+            f.write(data)
 
     def _get_unaggregated_timeserie(self, metric, version=3):
         path = self._build_unaggregated_timeserie_path(metric, version)
@@ -98,17 +105,7 @@ class FileStorage(_carbonara.CarbonaraBasedStorage):
                 raise storage.MetricDoesNotExist(metric)
             raise
 
-    def _delete_unaggregated_timeserie(self, metric, version=3):
-        path = self._build_unaggregated_timeserie_path(metric, version)
-        try:
-            os.unlink(path)
-        except IOError as e:
-            if e.errno == errno.ENOENT:
-                raise storage.MetricDoesNotExist(metric)
-            raise
-
-    def _list_split_keys_for_metric(self, metric, aggregation, granularity,
-                                    version=None):
+    def _list_split_keys(self, metric, aggregation, granularity, version=3):
         try:
             files = os.listdir(self._build_metric_path(metric, aggregation))
         except OSError as e:
@@ -116,23 +113,22 @@ class FileStorage(_carbonara.CarbonaraBasedStorage):
                 raise storage.MetricDoesNotExist(metric)
             raise
         keys = set()
+        granularity = str(utils.timespan_total_seconds(granularity))
         for f in files:
             meta = f.split("_")
-            if meta[1] == str(granularity) and self._version_check(f, version):
+            if meta[1] == granularity and self._version_check(f, version):
                 keys.add(meta[0])
         return keys
 
-    def _delete_metric_measures(self, metric, timestamp_key, aggregation,
-                                granularity, version=3):
+    def _delete_metric_measures(self, metric, key, aggregation, version=3):
         os.unlink(self._build_metric_path_for_split(
-            metric, aggregation, timestamp_key, granularity, version))
+            metric, aggregation, key, version))
 
-    def _store_metric_measures(self, metric, timestamp_key, aggregation,
-                               granularity, data, offset=None, version=3):
+    def _store_metric_measures(self, metric, key, aggregation,
+                               data, offset=None, version=3):
         self._atomic_file_store(
-            self._build_metric_path_for_split(metric, aggregation,
-                                              timestamp_key, granularity,
-                                              version),
+            self._build_metric_path_for_split(
+                metric, aggregation, key, version),
             data)
 
     def _delete_metric(self, metric):
@@ -145,16 +141,16 @@ class FileStorage(_carbonara.CarbonaraBasedStorage):
                 # measures)
                 raise
 
-    def _get_measures(self, metric, timestamp_key, aggregation, granularity,
-                      version=3):
+    def _get_measures_unbatched(self, metric, key, aggregation, version=3):
         path = self._build_metric_path_for_split(
-            metric, aggregation, timestamp_key, granularity, version)
+            metric, aggregation, key, version)
         try:
             with open(path, 'rb') as aggregation_file:
                 return aggregation_file.read()
         except IOError as e:
             if e.errno == errno.ENOENT:
                 if os.path.exists(self._build_metric_dir(metric)):
-                    raise storage.AggregationDoesNotExist(metric, aggregation)
+                    raise storage.AggregationDoesNotExist(
+                        metric, aggregation, key.sampling)
                 raise storage.MetricDoesNotExist(metric)
             raise
