@@ -15,29 +15,36 @@
 # under the License.
 
 from oslo_config import cfg
-from oslo_log import log
 
+from gnocchi.common import swift
 from gnocchi import storage
-from gnocchi.storage import _carbonara
-from gnocchi.storage.common import swift
+from gnocchi import utils
 
 swclient = swift.swclient
 swift_utils = swift.swift_utils
 
-LOG = log.getLogger(__name__)
-
 OPTS = [
     cfg.StrOpt('swift_auth_version',
                default='1',
                help='Swift authentication version to user.'),
-    cfg.StrOpt('swift_preauthurl',
-               help='Swift pre-auth URL.'),
+    cfg.BoolOpt('swift_auth_insecure',
+                default=False,
+                help='If True, swiftclient won\'t check for a valid SSL '
+                     'certificate when authenticating.'),
+    cfg.StrOpt('swift_url',
+               help='Swift URL. '
+               'If unset, it is obtained from the auth service.'),
     cfg.StrOpt('swift_authurl',
                default="http://localhost:8080/auth/v1.0",
                help='Swift auth URL.'),
     cfg.StrOpt('swift_preauthtoken',
                secret=True,
                help='Swift token to user to authenticate.'),
+    cfg.StrOpt('swift_cacert',
+               help='A string giving the CA certificate file to use in '
+                    'SSL connections for verifying certs.'),
+    cfg.StrOpt('swift_region',
+               help='Swift region.'),
     cfg.StrOpt('swift_user',
                default="admin:admin",
                help='Swift user.'),
@@ -60,6 +67,11 @@ OPTS = [
     cfg.StrOpt('swift_endpoint_type',
                default='publicURL',
                help='Endpoint type to connect to Swift',),
+    cfg.StrOpt('swift_service_type',
+               default='object-store',
+               help='A string giving the service type of the swift service '
+                    'to use. This setting is only used if '
+                    'swift_auth_version is 2.'),
     cfg.IntOpt('swift_timeout',
                min=0,
                default=300,
@@ -67,21 +79,27 @@ OPTS = [
 ]
 
 
-class SwiftStorage(_carbonara.CarbonaraBasedStorage):
+class SwiftStorage(storage.StorageDriver):
 
     WRITE_FULL = True
 
-    def __init__(self, conf, incoming):
-        super(SwiftStorage, self).__init__(conf, incoming)
+    def __init__(self, conf, coord=None):
+        super(SwiftStorage, self).__init__(conf, coord)
         self.swift = swift.get_connection(conf)
         self._container_prefix = conf.swift_container_prefix
 
+    def __str__(self):
+        return "%s: %s" % (self.__class__.__name__, self._container_prefix)
+
     def _container_name(self, metric):
         return '%s.%s' % (self._container_prefix, str(metric.id))
 
     @staticmethod
-    def _object_name(split_key, aggregation, granularity, version=3):
-        name = '%s_%s_%s' % (split_key, aggregation, granularity)
+    def _object_name(split_key, aggregation, version=3):
+        name = '%s_%s_%s' % (
+            split_key, aggregation,
+            utils.timespan_total_seconds(split_key.sampling),
+        )
         return name + '_v%s' % version if version else name
 
     def _create_metric(self, metric):
@@ -94,23 +112,19 @@ class SwiftStorage(_carbonara.CarbonaraBasedStorage):
         if resp['status'] == 204:
             raise storage.MetricAlreadyExists(metric)
 
-    def _store_metric_measures(self, metric, timestamp_key, aggregation,
-                               granularity, data, offset=None, version=3):
+    def _store_metric_measures(self, metric, key, aggregation,
+                               data, offset=None, version=3):
         self.swift.put_object(
             self._container_name(metric),
-            self._object_name(timestamp_key, aggregation, granularity,
-                              version),
+            self._object_name(key, aggregation, version),
             data)
 
-    def _delete_metric_measures(self, metric, timestamp_key, aggregation,
-                                granularity, version=3):
+    def _delete_metric_measures(self, metric, key, aggregation, version=3):
         self.swift.delete_object(
             self._container_name(metric),
-            self._object_name(timestamp_key, aggregation, granularity,
-                              version))
+            self._object_name(key, aggregation, version))
 
     def _delete_metric(self, metric):
-        self._delete_unaggregated_timeserie(metric)
         container = self._container_name(metric)
         try:
             headers, files = self.swift.get_container(
@@ -128,12 +142,11 @@ class SwiftStorage(_carbonara.CarbonaraBasedStorage):
                     # Deleted in the meantime? Whatever.
                     raise
 
-    def _get_measures(self, metric, timestamp_key, aggregation, granularity,
-                      version=3):
+    def _get_measures_unbatched(self, metric, key, aggregation, version=3):
         try:
             headers, contents = self.swift.get_object(
                 self._container_name(metric), self._object_name(
-                    timestamp_key, aggregation, granularity, version))
+                    key, aggregation, version))
         except swclient.ClientException as e:
             if e.http_status == 404:
                 try:
@@ -142,12 +155,12 @@ class SwiftStorage(_carbonara.CarbonaraBasedStorage):
                     if e.http_status == 404:
                         raise storage.MetricDoesNotExist(metric)
                     raise
-                raise storage.AggregationDoesNotExist(metric, aggregation)
+                raise storage.AggregationDoesNotExist(
+                    metric, aggregation, key.sampling)
             raise
         return contents
 
-    def _list_split_keys_for_metric(self, metric, aggregation, granularity,
-                                    version=None):
+    def _list_split_keys(self, metric, aggregation, granularity, version=3):
         container = self._container_name(metric)
         try:
             headers, files = self.swift.get_container(
@@ -157,10 +170,11 @@ class SwiftStorage(_carbonara.CarbonaraBasedStorage):
                 raise storage.MetricDoesNotExist(metric)
             raise
         keys = set()
+        granularity = str(utils.timespan_total_seconds(granularity))
         for f in files:
             try:
                 meta = f['name'].split('_')
-                if (aggregation == meta[1] and granularity == float(meta[2])
+                if (aggregation == meta[1] and granularity == meta[2]
                         and self._version_check(f['name'], version)):
                     keys.add(meta[0])
             except (ValueError, IndexError):
@@ -187,12 +201,3 @@ class SwiftStorage(_carbonara.CarbonaraBasedStorage):
         self.swift.put_object(self._container_name(metric),
                               self._build_unaggregated_timeserie_path(version),
                               data)
-
-    def _delete_unaggregated_timeserie(self, metric, version=3):
-        try:
-            self.swift.delete_object(
-                self._container_name(metric),
-                self._build_unaggregated_timeserie_path(version))
-        except swclient.ClientException as e:
-            if e.http_status != 404:
-                raise
