@@ -15,12 +15,15 @@
 # under the License.
 import functools
 import json
+import logging
 import os
 import subprocess
+import threading
 import uuid
 
+import daiquiri
 import fixtures
-from oslotest import base
+import numpy
 import six
 from six.moves.urllib.parse import unquote
 try:
@@ -28,13 +31,15 @@ try:
 except ImportError:
     swexc = None
 from testtools import testcase
-from tooz import coordination
 
 from gnocchi import archive_policy
+from gnocchi.cli import metricd
 from gnocchi import exceptions
+from gnocchi import incoming
 from gnocchi import indexer
 from gnocchi import service
 from gnocchi import storage
+from gnocchi.tests import utils
 
 
 class SkipNotImplementedMeta(type):
@@ -172,8 +177,43 @@ class FakeSwiftClient(object):
         return {}, None
 
 
+class CaptureOutput(fixtures.Fixture):
+    """Optionally capture the output streams.
+
+    .. py:attribute:: stdout
+
+       The ``stream`` attribute from a :class:`StringStream` instance
+       replacing stdout.
+
+    .. py:attribute:: stderr
+
+       The ``stream`` attribute from a :class:`StringStream` instance
+       replacing stderr.
+
+    """
+
+    def setUp(self):
+        super(CaptureOutput, self).setUp()
+        self._stdout_fixture = fixtures.StringStream('stdout')
+        self.stdout = self.useFixture(self._stdout_fixture).stream
+        self.useFixture(fixtures.MonkeyPatch('sys.stdout', self.stdout))
+        self._stderr_fixture = fixtures.StringStream('stderr')
+        self.stderr = self.useFixture(self._stderr_fixture).stream
+        self.useFixture(fixtures.MonkeyPatch('sys.stderr', self.stderr))
+
+
+class BaseTestCase(testcase.TestCase):
+    def setUp(self):
+        super(BaseTestCase, self).setUp()
+        if not os.getenv("GNOCCHI_TEST_DEBUG"):
+            self.useFixture(CaptureOutput())
+
+
 @six.add_metaclass(SkipNotImplementedMeta)
-class TestCase(base.BaseTestCase):
+class TestCase(BaseTestCase):
+
+    REDIS_DB_INDEX = 0
+    REDIS_DB_LOCK = threading.Lock()
 
     ARCHIVE_POLICIES = {
         'no_granularity_match': archive_policy.ArchivePolicy(
@@ -181,46 +221,48 @@ class TestCase(base.BaseTestCase):
             0, [
                 # 2 second resolution for a day
                 archive_policy.ArchivePolicyItem(
-                    granularity=2, points=3600 * 24),
+                    granularity=numpy.timedelta64(2, 's'),
+                    timespan=numpy.timedelta64(1, 'D'),
+                ),
             ],
         ),
         'low': archive_policy.ArchivePolicy(
             "low", 0, [
                 # 5 minutes resolution for an hour
                 archive_policy.ArchivePolicyItem(
-                    granularity=300, points=12),
+                    granularity=numpy.timedelta64(5, 'm'), points=12),
                 # 1 hour resolution for a day
                 archive_policy.ArchivePolicyItem(
-                    granularity=3600, points=24),
+                    granularity=numpy.timedelta64(1, 'h'), points=24),
                 # 1 day resolution for a month
                 archive_policy.ArchivePolicyItem(
-                    granularity=3600 * 24, points=30),
+                    granularity=numpy.timedelta64(1, 'D'), points=30),
             ],
         ),
         'medium': archive_policy.ArchivePolicy(
             "medium", 0, [
                 # 1 minute resolution for an day
                 archive_policy.ArchivePolicyItem(
-                    granularity=60, points=60 * 24),
+                    granularity=numpy.timedelta64(1, 'm'), points=60 * 24),
                 # 1 hour resolution for a week
                 archive_policy.ArchivePolicyItem(
-                    granularity=3600, points=7 * 24),
+                    granularity=numpy.timedelta64(1, 'h'), points=7 * 24),
                 # 1 day resolution for a year
                 archive_policy.ArchivePolicyItem(
-                    granularity=3600 * 24, points=365),
+                    granularity=numpy.timedelta64(1, 'D'), points=365),
             ],
         ),
         'high': archive_policy.ArchivePolicy(
             "high", 0, [
                 # 1 second resolution for an hour
                 archive_policy.ArchivePolicyItem(
-                    granularity=1, points=3600),
+                    granularity=numpy.timedelta64(1, 's'), points=3600),
                 # 1 minute resolution for a week
                 archive_policy.ArchivePolicyItem(
-                    granularity=60, points=60 * 24 * 7),
+                    granularity=numpy.timedelta64(1, 'm'), points=60 * 24 * 7),
                 # 1 hour resolution for a year
                 archive_policy.ArchivePolicyItem(
-                    granularity=3600, points=365 * 24),
+                    granularity=numpy.timedelta64(1, 'h'), points=365 * 24),
             ],
         ),
     }
@@ -228,8 +270,23 @@ class TestCase(base.BaseTestCase):
     @classmethod
     def setUpClass(self):
         super(TestCase, self).setUpClass()
-        self.conf = service.prepare_service([],
-                                            default_config_files=[])
+
+        self.conf = service.prepare_service(
+            [], conf=utils.prepare_conf(),
+            default_config_files=[],
+            logging_level=logging.DEBUG)
+
+        if not os.getenv("GNOCCHI_TEST_DEBUG"):
+            daiquiri.setup(outputs=[])
+
+        py_root = os.path.abspath(os.path.join(os.path.dirname(__file__),
+                                               '..',))
+        self.conf.set_override('paste_config',
+                               os.path.join(py_root, 'rest', 'api-paste.ini'),
+                               group="api")
+        self.conf.set_override('policy_file',
+                               os.path.join(py_root, 'rest', 'policy.json'),
+                               group="oslo_policy")
 
         # NOTE(jd) This allows to test S3 on AWS
         if not os.getenv("AWS_ACCESS_KEY_ID"):
@@ -242,24 +299,19 @@ class TestCase(base.BaseTestCase):
                                    group="storage")
 
         self.index = indexer.get_driver(self.conf)
-        self.index.connect()
+
+        self.coord = metricd.get_coordinator_and_start(
+            str(uuid.uuid4()),
+            self.conf.coordination_url)
 
         # NOTE(jd) So, some driver, at least SQLAlchemy, can't create all
         # their tables in a single transaction even with the
         # checkfirst=True, so what we do here is we force the upgrade code
         # path to be sequential to avoid race conditions as the tests run
         # in parallel.
-        self.coord = coordination.get_coordinator(
-            self.conf.storage.coordination_url,
-            str(uuid.uuid4()).encode('ascii'))
-
-        self.coord.start(start_heart=True)
-
         with self.coord.get_lock(b"gnocchi-tests-db-lock"):
             self.index.upgrade()
 
-        self.coord.stop()
-
         self.archive_policies = self.ARCHIVE_POLICIES.copy()
         for name, ap in six.iteritems(self.archive_policies):
             # Create basic archive policies
@@ -289,8 +341,10 @@ class TestCase(base.BaseTestCase):
                                    'storage')
         elif self.conf.storage.driver == 'ceph':
             pool_name = uuid.uuid4().hex
-            subprocess.call("rados -c %s mkpool %s" % (
-                os.getenv("CEPH_CONF"), pool_name), shell=True)
+            with open(os.devnull, 'w') as f:
+                subprocess.call("rados -c %s mkpool %s" % (
+                    os.getenv("CEPH_CONF"), pool_name), shell=True,
+                    stdout=f, stderr=subprocess.STDOUT)
             self.conf.set_override('ceph_pool', pool_name, 'storage')
 
         # Override the bucket prefix to be unique to avoid concurrent access
@@ -298,15 +352,38 @@ class TestCase(base.BaseTestCase):
         self.conf.set_override("s3_bucket_prefix", str(uuid.uuid4())[:26],
                                "storage")
 
-        self.storage = storage.get_driver(self.conf)
-        # NOTE(jd) Do not upgrade the storage. We don't really need the storage
-        # upgrade for now, and the code that upgrade from pre-1.3
-        # (TimeSerieArchive) uses a lot of parallel lock, which makes tooz
-        # explodes because MySQL does not support that many connections in real
-        # life.
-        # self.storage.upgrade(self.index)
+        self.storage = storage.get_driver(self.conf, self.coord)
+        self.incoming = incoming.get_driver(self.conf)
+
+        if self.conf.storage.driver == 'redis':
+            # Create one prefix per test
+            self.storage.STORAGE_PREFIX = str(uuid.uuid4()).encode()
+
+        if self.conf.incoming.driver == 'redis':
+            self.incoming.SACK_PREFIX = str(uuid.uuid4())
+
+        self.storage.upgrade()
+        self.incoming.upgrade(128)
 
     def tearDown(self):
         self.index.disconnect()
-        self.storage.stop()
         super(TestCase, self).tearDown()
+
+    @classmethod
+    def tearDownClass(cls):
+        cls.coord.stop()
+        super(TestCase, cls).tearDownClass()
+
+    def _create_metric(self, archive_policy_name="low"):
+        """Create a metric and return it"""
+        m = indexer.Metric(uuid.uuid4(),
+                           self.archive_policies[archive_policy_name])
+        m_sql = self.index.create_metric(m.id, str(uuid.uuid4()),
+                                         archive_policy_name)
+        return m, m_sql
+
+    def trigger_processing(self, metrics=None):
+        if metrics is None:
+            metrics = [str(self.metric.id)]
+        self.storage.process_new_measures(
+            self.index, self.incoming, metrics, sync=True)
