@@ -18,6 +18,7 @@ import operator
 import uuid
 
 import mock
+import numpy
 
 from gnocchi import archive_policy
 from gnocchi import indexer
@@ -37,6 +38,10 @@ class TestIndexer(tests_base.TestCase):
 
 class TestIndexerDriver(tests_base.TestCase):
 
+    def test_str(self):
+        self.assertEqual("%s: %s" % (self.index.__class__.__name__,
+                         self.conf.indexer.url), str(self.index))
+
     def test_create_archive_policy_already_exists(self):
         # NOTE(jd) This archive policy
         # is created by gnocchi.tests on setUp() :)
@@ -51,9 +56,15 @@ class TestIndexerDriver(tests_base.TestCase):
             'aggregation_methods':
             set(self.conf.archive_policy.default_aggregation_methods),
             'definition': [
-                {u'granularity': 300, u'points': 12, u'timespan': 3600},
-                {u'granularity': 3600, u'points': 24, u'timespan': 86400},
-                {u'granularity': 86400, u'points': 30, u'timespan': 2592000}],
+                {u'granularity': numpy.timedelta64(5, 'm'),
+                 u'points': 12,
+                 u'timespan': numpy.timedelta64(3600, 's')},
+                {u'granularity': numpy.timedelta64(3600, 's'),
+                 u'points': 24,
+                 u'timespan': numpy.timedelta64(86400, 's')},
+                {u'granularity': numpy.timedelta64(86400, 's'),
+                 u'points': 30,
+                 u'timespan': numpy.timedelta64(2592000, 's')}],
             'name': u'low'}, dict(ap))
 
     def test_update_archive_policy(self):
@@ -84,9 +95,15 @@ class TestIndexerDriver(tests_base.TestCase):
             'aggregation_methods':
             set(self.conf.archive_policy.default_aggregation_methods),
             'definition': [
-                {u'granularity': 300, u'points': 6, u'timespan': 1800},
-                {u'granularity': 3600, u'points': 24, u'timespan': 86400},
-                {u'granularity': 86400, u'points': 30, u'timespan': 2592000}],
+                {u'granularity': numpy.timedelta64(300, 's'),
+                 u'points': 6,
+                 u'timespan': numpy.timedelta64(1800, 's')},
+                {u'granularity': numpy.timedelta64(3600, 's'),
+                 u'points': 24,
+                 u'timespan': numpy.timedelta64(86400, 's')},
+                {u'granularity': numpy.timedelta64(86400, 's'),
+                 u'points': 30,
+                 u'timespan': numpy.timedelta64(2592000, 's')}],
             'name': apname}, dict(ap))
         ap = self.index.update_archive_policy(
             apname, [archive_policy.ArchivePolicyItem(granularity=300,
@@ -100,9 +117,15 @@ class TestIndexerDriver(tests_base.TestCase):
             'aggregation_methods':
             set(self.conf.archive_policy.default_aggregation_methods),
             'definition': [
-                {u'granularity': 300, u'points': 12, u'timespan': 3600},
-                {u'granularity': 3600, u'points': 24, u'timespan': 86400},
-                {u'granularity': 86400, u'points': 30, u'timespan': 2592000}],
+                {u'granularity': numpy.timedelta64(300, 's'),
+                 u'points': 12,
+                 u'timespan': numpy.timedelta64(3600, 's')},
+                {u'granularity': numpy.timedelta64(3600, 's'),
+                 u'points': 24,
+                 u'timespan': numpy.timedelta64(86400, 's')},
+                {u'granularity': numpy.timedelta64(86400, 's'),
+                 u'points': 30,
+                 u'timespan': numpy.timedelta64(2592000, 's')}],
             'name': apname}, dict(ap))
 
     def test_delete_archive_policy(self):
@@ -156,7 +179,7 @@ class TestIndexerDriver(tests_base.TestCase):
         self.assertIsNone(m.name)
         self.assertIsNone(m.unit)
         self.assertIsNone(m.resource_id)
-        m2 = self.index.list_metrics(id=r1)
+        m2 = self.index.list_metrics(attribute_filter={"=": {"id": r1}})
         self.assertEqual([m], m2)
 
     def test_create_named_metric_duplicate(self):
@@ -172,7 +195,7 @@ class TestIndexerDriver(tests_base.TestCase):
         self.assertEqual(m.creator, creator)
         self.assertEqual(name, m.name)
         self.assertEqual(r1, m.resource_id)
-        m2 = self.index.list_metrics(id=m1)
+        m2 = self.index.list_metrics(attribute_filter={"=": {"id": m1}})
         self.assertEqual([m], m2)
 
         self.assertRaises(indexer.NamedMetricAlreadyExists,
@@ -299,7 +322,8 @@ class TestIndexerDriver(tests_base.TestCase):
             'generic', r1, creator,
             metrics={"foobar": {"archive_policy_name": "low"}})
         self.assertEqual(1, len(rc.metrics))
-        m = self.index.list_metrics(id=rc.metrics[0].id)
+        m = self.index.list_metrics(
+            attribute_filter={"=": {"id": rc.metrics[0].id}})
         self.assertEqual(m[0], rc.metrics[0])
 
     def test_delete_resource(self):
@@ -324,7 +348,8 @@ class TestIndexerDriver(tests_base.TestCase):
         self.assertRaises(indexer.NoSuchResource,
                           self.index.delete_resource,
                           r1)
-        metrics = self.index.list_metrics(ids=[e1, e2])
+        metrics = self.index.list_metrics(
+            attribute_filter={"in": {"id": [e1, e2]}})
         self.assertEqual([], metrics)
 
     def test_delete_resource_non_existent(self):
@@ -1074,8 +1099,9 @@ class TestIndexerDriver(tests_base.TestCase):
         g2 = self.index.create_resource('generic', r2, creator,
                                         user, project, metrics=metrics)
 
-        metrics = self.index.list_metrics(ids=[g1['metrics'][0]['id'],
-                                               g2['metrics'][0]['id']])
+        metrics = self.index.list_metrics(
+            attribute_filter={"in": {"id": [g1['metrics'][0]['id'],
+                                            g2['metrics'][0]['id']]}})
         self.assertEqual(2, len(metrics))
         for m in metrics:
             self.assertEqual('active', m['status'])
@@ -1090,9 +1116,10 @@ class TestIndexerDriver(tests_base.TestCase):
             attribute_filter={"=": {"user_id": user}})
         self.assertEqual(0, len(resources))
 
-        metrics = self.index.list_metrics(ids=[g1['metrics'][0]['id'],
-                                               g2['metrics'][0]['id']],
-                                          status='delete')
+        metrics = self.index.list_metrics(
+            attribute_filter={"in": {"id": [g1['metrics'][0]['id'],
+                                            g2['metrics'][0]['id']]}},
+            status='delete')
         self.assertEqual(2, len(metrics))
         for m in metrics:
             self.assertEqual('delete', m['status'])
@@ -1102,7 +1129,7 @@ class TestIndexerDriver(tests_base.TestCase):
         creator = str(uuid.uuid4())
         self.index.create_metric(e1, creator, archive_policy_name="low")
 
-        metric = self.index.list_metrics(id=e1)
+        metric = self.index.list_metrics(attribute_filter={"=": {"id": e1}})
         self.assertEqual(1, len(metric))
         metric = metric[0]
         self.assertEqual(e1, metric.id)
@@ -1117,7 +1144,7 @@ class TestIndexerDriver(tests_base.TestCase):
                                  creator,
                                  archive_policy_name="low")
 
-        metric = self.index.list_metrics(id=e1)
+        metric = self.index.list_metrics(attribute_filter={"=": {"id": e1}})
         self.assertEqual(1, len(metric))
         metric = metric[0]
         self.assertEqual(e1, metric.id)
@@ -1128,10 +1155,12 @@ class TestIndexerDriver(tests_base.TestCase):
 
     def test_get_metric_with_bad_uuid(self):
         e1 = uuid.uuid4()
-        self.assertEqual([], self.index.list_metrics(id=e1))
+        self.assertEqual([], self.index.list_metrics(
+            attribute_filter={"=": {"id": e1}}))
 
     def test_get_metric_empty_list_uuids(self):
-        self.assertEqual([], self.index.list_metrics(ids=[]))
+        self.assertEqual([], self.index.list_metrics(
+            attribute_filter={"in": {"id": []}}))
 
     def test_list_metrics(self):
         e1 = uuid.uuid4()
@@ -1148,6 +1177,23 @@ class TestIndexerDriver(tests_base.TestCase):
         else:
             self.assertLess(id_list.index(e2), id_list.index(e1))
 
+    def test_list_metrics_resource_filter(self):
+        r1 = uuid.uuid4()
+        creator = str(uuid.uuid4())
+        m1 = uuid.uuid4()
+        m2 = uuid.uuid4()
+        project_id = str(uuid.uuid4())
+        self.index.create_resource("generic", r1, creator,
+                                   project_id=project_id)
+        self.index.create_metric(m1, creator, archive_policy_name="low",
+                                 resource_id=r1)
+        self.index.create_metric(m2, creator, archive_policy_name="low")
+        metrics = self.index.list_metrics(
+            resource_policy_filter={"=": {"project_id": project_id}})
+        id_list = [m.id for m in metrics]
+        self.assertIn(m1, id_list)
+        self.assertNotIn(m2, id_list)
+
     def test_list_metrics_delete_status(self):
         e1 = uuid.uuid4()
         self.index.create_metric(e1, str(uuid.uuid4()),
