@@ -23,19 +23,21 @@ import hashlib
 import json
 import uuid
 
+import fixtures
+import iso8601
 from keystonemiddleware import fixture as ksm_fixture
 import mock
-from oslo_utils import timeutils
+import pbr.version
 import six
-from stevedore import extension
 import testscenarios
 from testtools import testcase
 import webtest
 
 from gnocchi import archive_policy
-from gnocchi import rest
+from gnocchi.rest import api
 from gnocchi.rest import app
 from gnocchi.tests import base as tests_base
+from gnocchi.tests import utils as tests_utils
 from gnocchi import utils
 
 
@@ -61,6 +63,7 @@ class TestingApp(webtest.TestApp):
         self.auth_mode = kwargs.pop('auth_mode')
         self.storage = kwargs.pop('storage')
         self.indexer = kwargs.pop('indexer')
+        self.incoming = kwargs.pop('incoming')
         super(TestingApp, self).__init__(*args, **kwargs)
         # Setup Keystone auth_token fake cache
         self.token = self.VALID_TOKEN
@@ -83,8 +86,13 @@ class TestingApp(webtest.TestApp):
                 yield
             finally:
                 self.user = old_user
-        elif self.auth_mode == "noauth":
-            raise testcase.TestSkipped("auth mode is noauth")
+        elif self.auth_mode == "remoteuser":
+            old_user = self.user
+            self.user = b"admin"
+            try:
+                yield
+            finally:
+                self.user = old_user
         else:
             raise RuntimeError("Unknown auth_mode")
 
@@ -118,13 +126,12 @@ class TestingApp(webtest.TestApp):
             req.headers['Authorization'] = (
                 b"basic " + base64.b64encode(self.user + b":")
             )
-        elif self.auth_mode == "noauth":
-            req.headers['X-User-Id'] = self.USER_ID
-            req.headers['X-Project-Id'] = self.PROJECT_ID
+        elif self.auth_mode == "remoteuser":
+            req.remote_user = self.user
         response = super(TestingApp, self).do_request(req, *args, **kwargs)
-        metrics = self.storage.incoming.list_metric_with_measures_to_process(
-            None, None, full=True)
-        self.storage.process_background_tasks(self.indexer, metrics, sync=True)
+        metrics = tests_utils.list_all_incoming_metrics(self.incoming)
+        self.storage.process_new_measures(
+            self.indexer, self.incoming, metrics, sync=True)
         return response
 
 
@@ -133,7 +140,7 @@ class RestTest(tests_base.TestCase, testscenarios.TestWithScenarios):
     scenarios = [
         ('basic', dict(auth_mode="basic")),
         ('keystone', dict(auth_mode="keystone")),
-        ('noauth', dict(auth_mode="noauth")),
+        ('remoteuser', dict(auth_mode="remoteuser")),
     ]
 
     def setUp(self):
@@ -166,14 +173,28 @@ class RestTest(tests_base.TestCase, testscenarios.TestWithScenarios):
 
         self.conf.set_override("auth_mode", self.auth_mode, group="api")
 
+        self.useFixture(fixtures.MockPatchObject(
+            app.GnocchiHook, "_lazy_load", self._fake_lazy_load))
+
         self.app = TestingApp(app.load_app(conf=self.conf,
-                                           indexer=self.index,
-                                           storage=self.storage,
                                            not_implemented_middleware=False),
                               storage=self.storage,
                               indexer=self.index,
+                              incoming=self.incoming,
                               auth_mode=self.auth_mode)
 
+    def _fake_lazy_load(self, name):
+        if name == "storage":
+            return self.storage
+        elif name == "indexer":
+            return self.index
+        elif name == "incoming":
+            return self.incoming
+        elif name == "coordinator":
+            return self.coord
+        else:
+            raise RuntimeError("Invalid driver type: %s" % name)
+
     # NOTE(jd) Used at least by docs
     @staticmethod
     def runTest():
@@ -189,21 +210,19 @@ class RootTest(RestTest):
                 status=415)
 
     def test_capabilities(self):
-        custom_agg = extension.Extension('test_aggregation', None, None, None)
-        mgr = extension.ExtensionManager.make_test_instance(
-            [custom_agg], 'gnocchi.aggregates')
         aggregation_methods = set(
             archive_policy.ArchivePolicy.VALID_AGGREGATION_METHODS)
+        result = self.app.get("/v1/capabilities").json
+        self.assertEqual(
+            sorted(aggregation_methods),
+            sorted(result['aggregation_methods']))
 
-        with mock.patch.object(extension, 'ExtensionManager',
-                               return_value=mgr):
-            result = self.app.get("/v1/capabilities").json
-            self.assertEqual(
-                sorted(aggregation_methods),
-                sorted(result['aggregation_methods']))
-            self.assertEqual(
-                ['test_aggregation'],
-                result['dynamic_aggregation_methods'])
+    def test_version(self):
+        with self.app.use_admin_user():
+            r = self.app.get("/")
+        self.assertEqual(
+            json.loads(r.text)['build'],
+            pbr.version.VersionInfo('gnocchi').version_string())
 
     def test_status(self):
         with self.app.use_admin_user():
@@ -319,6 +338,22 @@ class MetricTest(RestTest):
             metric_list = self.app.get("/v1/metric")
             self.assertNotIn(metric_id, [m["id"] for m in metric_list.json])
 
+    def test_list_metric_with_another_user_allowed(self):
+        rid = str(uuid.uuid4())
+        r = self.app.post_json("/v1/resource/generic",
+                               params={
+                                   "id": rid,
+                                   "project_id": TestingApp.PROJECT_ID_2,
+                                   "metrics": {
+                                       "disk": {"archive_policy_name": "low"},
+                                   }
+                               })
+        metric_id = r.json['metrics']['disk']
+
+        with self.app.use_another_user():
+            metric_list = self.app.get("/v1/metric")
+            self.assertIn(metric_id, [m["id"] for m in metric_list.json])
+
     def test_get_metric_with_another_user(self):
         result = self.app.post_json("/v1/metric",
                                     params={"archive_policy_name": "medium"},
@@ -429,7 +464,56 @@ class MetricTest(RestTest):
             self.app.get("/v1/metric/%s/measures" % metric['id'],
                          status=403)
 
-    @mock.patch.object(timeutils, 'utcnow')
+    def test_get_measures_with_another_user_allowed(self):
+        rid = str(uuid.uuid4())
+        result = self.app.post_json(
+            "/v1/resource/generic",
+            params={
+                "id": rid,
+                "project_id": TestingApp.PROJECT_ID_2,
+                "metrics": {
+                    "disk": {"archive_policy_name": "low"},
+                }
+            })
+        metric_id = result.json['metrics']['disk']
+        measures_url = "/v1/resource/generic/%s/metric/disk/measures" % rid
+        self.app.post_json(measures_url,
+                           params=[{"timestamp": '2013-01-01 23:23:23',
+                                    "value": 1234.2}])
+        with self.app.use_another_user():
+            result = self.app.get(measures_url)
+            self.assertEqual(
+                [['2013-01-01T00:00:00+00:00', 86400.0, 1234.2],
+                 ['2013-01-01T23:00:00+00:00', 3600.0, 1234.2],
+                 ['2013-01-01T23:20:00+00:00', 300.0, 1234.2]],
+                result.json)
+            result = self.app.get("/v1/metric/%s/measures" % metric_id)
+            self.assertEqual(
+                [['2013-01-01T00:00:00+00:00', 86400.0, 1234.2],
+                 ['2013-01-01T23:00:00+00:00', 3600.0, 1234.2],
+                 ['2013-01-01T23:20:00+00:00', 300.0, 1234.2]],
+                result.json)
+
+    def test_get_measures_with_another_user_disallowed(self):
+        rid = str(uuid.uuid4())
+        result = self.app.post_json(
+            "/v1/resource/generic",
+            params={
+                "id": rid,
+                "metrics": {
+                    "disk": {"archive_policy_name": "low"},
+                }
+            })
+        metric_id = result.json['metrics']['disk']
+        measures_url = "/v1/resource/generic/%s/metric/disk/measures" % rid
+        self.app.post_json(measures_url,
+                           params=[{"timestamp": '2013-01-01 23:23:23',
+                                    "value": 1234.2}])
+        with self.app.use_another_user():
+            self.app.get(measures_url, status=403)
+            self.app.get("/v1/metric/%s/measures" % metric_id, status=403)
+
+    @mock.patch.object(utils, 'utcnow')
     def test_get_measure_start_relative(self, utcnow):
         """Make sure the timestamps can be relative to now."""
         utcnow.return_value = datetime.datetime(2014, 1, 1, 10, 23)
@@ -496,59 +580,6 @@ class MetricTest(RestTest):
                           [u'2013-01-01T12:00:00+00:00', 60.0, 12345.2]],
                          result)
 
-    def test_get_moving_average(self):
-        result = self.app.post_json("/v1/metric",
-                                    params={"archive_policy_name": "medium"})
-        metric = json.loads(result.text)
-        self.app.post_json("/v1/metric/%s/measures" % metric['id'],
-                           params=[{"timestamp": '2013-01-01 12:00:00',
-                                    "value": 69},
-                                   {"timestamp": '2013-01-01 12:00:20',
-                                    "value": 42},
-                                   {"timestamp": '2013-01-01 12:00:40',
-                                    "value": 6},
-                                   {"timestamp": '2013-01-01 12:01:00',
-                                    "value": 44},
-                                   {"timestamp": '2013-01-01 12:01:20',
-                                    "value": 7}])
-
-        path = "/v1/metric/%s/measures?aggregation=%s&window=%ds"
-        ret = self.app.get(path % (metric['id'], 'moving-average', 120),
-                           status=200)
-        result = json.loads(ret.text)
-        expected = [[u'2013-01-01T12:00:00+00:00', 120.0, 32.25]]
-        self.assertEqual(expected, result)
-        ret = self.app.get(path % (metric['id'], 'moving-average', 90),
-                           status=400)
-        self.assertIn('No data available that is either full-res',
-                      ret.text)
-        path = "/v1/metric/%s/measures?aggregation=%s"
-        ret = self.app.get(path % (metric['id'], 'moving-average'),
-                           status=400)
-        self.assertIn('Moving aggregate must have window specified',
-                      ret.text)
-
-    def test_get_moving_average_invalid_window(self):
-        result = self.app.post_json("/v1/metric",
-                                    params={"archive_policy_name": "medium"})
-        metric = json.loads(result.text)
-        self.app.post_json("/v1/metric/%s/measures" % metric['id'],
-                           params=[{"timestamp": '2013-01-01 12:00:00',
-                                    "value": 69},
-                                   {"timestamp": '2013-01-01 12:00:20',
-                                    "value": 42},
-                                   {"timestamp": '2013-01-01 12:00:40',
-                                    "value": 6},
-                                   {"timestamp": '2013-01-01 12:01:00',
-                                    "value": 44},
-                                   {"timestamp": '2013-01-01 12:01:20',
-                                    "value": 7}])
-
-        path = "/v1/metric/%s/measures?aggregation=%s&window=foobar"
-        ret = self.app.get(path % (metric['id'], 'moving-average'),
-                           status=400)
-        self.assertIn('Invalid value for window', ret.text)
-
     def test_get_resource_missing_named_metric_measure_aggregation(self):
         mgr = self.index.get_resource_type_schema()
         resource_type = str(uuid.uuid4())
@@ -652,12 +683,12 @@ class ResourceTest(RestTest):
         # Set original_resource_id
         self.resource['original_resource_id'] = self.resource['id']
         self.resource['created_by_user_id'] = TestingApp.USER_ID
-        if self.auth_mode in ("keystone", "noauth"):
+        if self.auth_mode == "keystone":
             self.resource['created_by_project_id'] = TestingApp.PROJECT_ID
             self.resource['creator'] = (
                 TestingApp.USER_ID + ":" + TestingApp.PROJECT_ID
             )
-        elif self.auth_mode == "basic":
+        elif self.auth_mode in ["basic", "remoteuser"]:
             self.resource['created_by_project_id'] = ""
             self.resource['creator'] = TestingApp.USER_ID
         self.resource['ended_at'] = None
@@ -744,7 +775,7 @@ class ResourceTest(RestTest):
     @staticmethod
     def _strtime_to_httpdate(dt):
         return email_utils.formatdate(calendar.timegm(
-            timeutils.parse_isotime(dt).timetuple()), usegmt=True)
+            iso8601.parse_date(dt).timetuple()), usegmt=True)
 
     def _check_etag(self, response, resource):
         lastmodified = self._strtime_to_httpdate(resource['revision_start'])
@@ -980,7 +1011,7 @@ class ResourceTest(RestTest):
         metrics = {'foo': {'archive_policy_name': "high"}}
         self.app.post_json("/v1/resource/" + self.resource_type
                            + "/" + self.attributes['id'] + "/metric",
-                           params=metrics, status=204)
+                           params=metrics, status=200)
         metrics = {'foo': {'archive_policy_name': "low"}}
         self.app.post_json("/v1/resource/" + self.resource_type
                            + "/" + self.attributes['id']
@@ -1001,7 +1032,7 @@ class ResourceTest(RestTest):
         metrics = {'foo': {'archive_policy_name': "high"}}
         self.app.post_json("/v1/resource/" + self.resource_type
                            + "/" + self.attributes['id'] + "/metric",
-                           params=metrics, status=204)
+                           params=metrics, status=200)
         result = self.app.get("/v1/resource/"
                               + self.resource_type + "/"
                               + self.attributes['id'])
@@ -1463,8 +1494,7 @@ class ResourceTest(RestTest):
         result = self.app.get("/v1/resource/generic?details=awesome",
                               status=400)
         self.assertIn(
-            b"Unable to parse details value in query: "
-            b"Unrecognized value 'awesome', acceptable values are",
+            b"Unable to parse `details': invalid truth value",
             result.body)
 
     def test_list_resources_with_bad_details_in_accept(self):
@@ -1474,8 +1504,7 @@ class ResourceTest(RestTest):
                               },
                               status=400)
         self.assertIn(
-            b"Unable to parse details value in Accept: "
-            b"Unrecognized value 'foo', acceptable values are",
+            b"Unable to parse `Accept header': invalid truth value",
             result.body)
 
     def _do_test_list_resources_with_detail(self, request):
@@ -1621,10 +1650,15 @@ class ResourceTest(RestTest):
             "/v1/aggregation/resource/"
             + self.resource_type + "/metric/foo?aggregation=max",
             params={"=": {"name": name}},
-            status=400)
-        self.assertIn(b"One of the metrics being aggregated doesn't have "
-                      b"matching granularity",
-                      result.body)
+            status=400,
+            headers={"Accept": "application/json"})
+        self.assertEqual("Metrics can't being aggregated",
+                         result.json['description']['cause'])
+        self.assertEqual("No granularity match",
+                         result.json['description']['reason'])
+        self.assertEqual(
+            sorted([[metric1['id'], 'max'], [metric2['id'], 'max']]),
+            sorted(result.json['description']['detail']))
 
     def test_get_res_named_metric_measure_aggregation_nooverlap(self):
         result = self.app.post_json("/v1/metric",
@@ -1660,17 +1694,22 @@ class ResourceTest(RestTest):
             + "/metric/foo?aggregation=max",
             params={"=": {"name": name}},
             expect_errors=True)
+        self.assertEqual(400, result.status_code, result.text)
+        self.assertIn("No overlap", result.text)
 
+        result = self.app.post_json(
+            "/v1/aggregation/resource/" + self.resource_type
+            + "/metric/foo?aggregation=max&needed_overlap=5&start=2013-01-01",
+            params={"=": {"name": name}},
+            expect_errors=True)
         self.assertEqual(400, result.status_code, result.text)
         self.assertIn("No overlap", result.text)
 
         result = self.app.post_json(
             "/v1/aggregation/resource/"
             + self.resource_type + "/metric/foo?aggregation=min"
-            + "&needed_overlap=0",
-            params={"=": {"name": name}},
-            expect_errors=True)
-
+            + "&needed_overlap=0&start=2013-01-01T00:00:00%2B00:00",
+            params={"=": {"name": name}})
         self.assertEqual(200, result.status_code, result.text)
         measures = json.loads(result.text)
         self.assertEqual([['2013-01-01T00:00:00+00:00', 86400.0, 8.0],
@@ -1861,7 +1900,7 @@ class GenericResourceTest(RestTest):
 
 class QueryStringSearchAttrFilterTest(tests_base.TestCase):
     def _do_test(self, expr, expected):
-        req = rest.QueryStringSearchAttrFilter.parse(expr)
+        req = api.QueryStringSearchAttrFilter._parse(expr)
         self.assertEqual(expected, req)
 
     def test_search_query_builder(self):
