@@ -66,12 +66,12 @@ class TestStorageDriver(tests_base.TestCase):
         self.assertIsInstance(driver, storage.StorageDriver)
 
     def test_corrupted_data(self):
-        self.incoming.add_measures(self.metric, [
+        self.incoming.add_measures(self.metric.id, [
             incoming.Measure(datetime64(2014, 1, 1, 12, 0, 1), 69),
         ])
         self.trigger_processing()
 
-        self.incoming.add_measures(self.metric, [
+        self.incoming.add_measures(self.metric.id, [
             incoming.Measure(datetime64(2014, 1, 1, 13, 0, 1), 1),
         ])
 
@@ -81,7 +81,13 @@ class TestStorageDriver(tests_base.TestCase):
                             side_effect=carbonara.InvalidData()):
                 self.trigger_processing()
 
-        m = self.storage.get_measures(self.metric)
+        granularities = [
+            numpy.timedelta64(1, 'D'),
+            numpy.timedelta64(1, 'h'),
+            numpy.timedelta64(5, 'm'),
+        ]
+
+        m = self.storage.get_measures(self.metric, granularities)
         self.assertIn((datetime64(2014, 1, 1),
                        numpy.timedelta64(1, 'D'), 1), m)
         self.assertIn((datetime64(2014, 1, 1, 13),
@@ -90,7 +96,7 @@ class TestStorageDriver(tests_base.TestCase):
                        numpy.timedelta64(5, 'm'), 1), m)
 
     def test_aborted_initial_processing(self):
-        self.incoming.add_measures(self.metric, [
+        self.incoming.add_measures(self.metric.id, [
             incoming.Measure(datetime64(2014, 1, 1, 12, 0, 1), 5),
         ])
         with mock.patch.object(self.storage, '_store_unaggregated_timeserie',
@@ -104,7 +110,13 @@ class TestStorageDriver(tests_base.TestCase):
             self.trigger_processing()
             self.assertFalse(LOG.error.called)
 
-        m = self.storage.get_measures(self.metric)
+        granularities = [
+            numpy.timedelta64(1, 'D'),
+            numpy.timedelta64(1, 'h'),
+            numpy.timedelta64(5, 'm'),
+        ]
+
+        m = self.storage.get_measures(self.metric, granularities)
         self.assertIn((datetime64(2014, 1, 1),
                        numpy.timedelta64(1, 'D'), 5.0), m)
         self.assertIn((datetime64(2014, 1, 1, 12),
@@ -115,7 +127,7 @@ class TestStorageDriver(tests_base.TestCase):
     def test_list_metric_with_measures_to_process(self):
         metrics = tests_utils.list_all_incoming_metrics(self.incoming)
         self.assertEqual(set(), metrics)
-        self.incoming.add_measures(self.metric, [
+        self.incoming.add_measures(self.metric.id, [
             incoming.Measure(datetime64(2014, 1, 1, 12, 0, 1), 69),
         ])
         metrics = tests_utils.list_all_incoming_metrics(self.incoming)
@@ -125,19 +137,23 @@ class TestStorageDriver(tests_base.TestCase):
         self.assertEqual(set([]), metrics)
 
     def test_delete_nonempty_metric(self):
-        self.incoming.add_measures(self.metric, [
+        self.incoming.add_measures(self.metric.id, [
             incoming.Measure(datetime64(2014, 1, 1, 12, 0, 1), 69),
         ])
         self.trigger_processing()
-        self.storage.delete_metric(self.incoming, self.metric, sync=True)
+        self.storage._delete_metric(self.metric)
         self.trigger_processing()
-        self.assertEqual([], self.storage.get_measures(self.metric))
+        self.assertEqual([], self.storage.get_measures(self.metric, [
+            numpy.timedelta64(1, 'D'),
+            numpy.timedelta64(1, 'h'),
+            numpy.timedelta64(5, 'm'),
+        ]))
         self.assertRaises(storage.MetricDoesNotExist,
                           self.storage._get_unaggregated_timeserie,
                           self.metric)
 
     def test_delete_nonempty_metric_unprocessed(self):
-        self.incoming.add_measures(self.metric, [
+        self.incoming.add_measures(self.metric.id, [
             incoming.Measure(datetime64(2014, 1, 1, 12, 0, 1), 69),
         ])
         self.index.delete_metric(self.metric.id)
@@ -149,7 +165,7 @@ class TestStorageDriver(tests_base.TestCase):
         self.assertNotIn(str(self.metric.id), details)
 
     def test_delete_expunge_metric(self):
-        self.incoming.add_measures(self.metric, [
+        self.incoming.add_measures(self.metric.id, [
             incoming.Measure(datetime64(2014, 1, 1, 12, 0, 1), 69),
         ])
         self.trigger_processing()
@@ -176,10 +192,10 @@ class TestStorageDriver(tests_base.TestCase):
     def test_measures_reporting(self):
         m2, __ = self._create_metric('medium')
         for i in six.moves.range(60):
-            self.incoming.add_measures(self.metric, [
+            self.incoming.add_measures(self.metric.id, [
                 incoming.Measure(datetime64(2014, 1, 1, 12, 0, i), 69),
             ])
-            self.incoming.add_measures(m2, [
+            self.incoming.add_measures(m2.id, [
                 incoming.Measure(datetime64(2014, 1, 1, 12, 0, i), 69),
             ])
         report = self.incoming.measures_report(True)
@@ -195,12 +211,16 @@ class TestStorageDriver(tests_base.TestCase):
 
     def test_add_measures_big(self):
         m, __ = self._create_metric('high')
-        self.incoming.add_measures(m, [
+        self.incoming.add_measures(m.id, [
             incoming.Measure(datetime64(2014, 1, 1, 12, i, j), 100)
             for i in six.moves.range(0, 60) for j in six.moves.range(0, 60)])
         self.trigger_processing([str(m.id)])
 
-        self.assertEqual(3661, len(self.storage.get_measures(m)))
+        self.assertEqual(3661, len(self.storage.get_measures(m, [
+            numpy.timedelta64(1, 'h'),
+            numpy.timedelta64(1, 'm'),
+            numpy.timedelta64(1, 's'),
+        ])))
 
     @mock.patch('gnocchi.carbonara.SplitKey.POINTS_PER_SPLIT', 48)
     def test_add_measures_update_subset_split(self):
@@ -208,11 +228,11 @@ class TestStorageDriver(tests_base.TestCase):
         measures = [
             incoming.Measure(datetime64(2014, 1, 6, i, j, 0), 100)
             for i in six.moves.range(2) for j in six.moves.range(0, 60, 2)]
-        self.incoming.add_measures(m, measures)
+        self.incoming.add_measures(m.id, measures)
         self.trigger_processing([str(m.id)])
 
         # add measure to end, in same aggregate time as last point.
-        self.incoming.add_measures(m, [
+        self.incoming.add_measures(m.id, [
             incoming.Measure(datetime64(2014, 1, 6, 1, 58, 1), 100)])
 
         with mock.patch.object(self.storage, '_store_metric_measures') as c:
@@ -233,13 +253,12 @@ class TestStorageDriver(tests_base.TestCase):
         measures = [
             incoming.Measure(datetime64(2014, 1, 6, i, j, 0), 100)
             for i in six.moves.range(2) for j in six.moves.range(0, 60, 2)]
-        self.incoming.add_measures(m, measures)
+        self.incoming.add_measures(m.id, measures)
         self.trigger_processing([str(m.id)])
 
         # add measure to end, in same aggregate time as last point.
         new_point = datetime64(2014, 1, 6, 1, 58, 1)
-        self.incoming.add_measures(
-            m, [incoming.Measure(new_point, 100)])
+        self.incoming.add_measures(m.id, [incoming.Measure(new_point, 100)])
 
         with mock.patch.object(self.incoming, 'add_measures') as c:
             self.trigger_processing([str(m.id)])
@@ -249,7 +268,7 @@ class TestStorageDriver(tests_base.TestCase):
                     new_point, args[1].granularity * 10e8))
 
     def test_delete_old_measures(self):
-        self.incoming.add_measures(self.metric, [
+        self.incoming.add_measures(self.metric.id, [
             incoming.Measure(datetime64(2014, 1, 1, 12, 0, 1), 69),
             incoming.Measure(datetime64(2014, 1, 1, 12, 7, 31), 42),
             incoming.Measure(datetime64(2014, 1, 1, 12, 9, 31), 4),
@@ -257,26 +276,31 @@ class TestStorageDriver(tests_base.TestCase):
         ])
         self.trigger_processing()
 
+        granularities = [
+            numpy.timedelta64(1, 'D'),
+            numpy.timedelta64(1, 'h'),
+            numpy.timedelta64(5, 'm'),
+        ]
+
         self.assertEqual([
             (datetime64(2014, 1, 1), numpy.timedelta64(1, 'D'), 39.75),
             (datetime64(2014, 1, 1, 12), numpy.timedelta64(1, 'h'), 39.75),
             (datetime64(2014, 1, 1, 12), numpy.timedelta64(5, 'm'), 69.0),
             (datetime64(2014, 1, 1, 12, 5), numpy.timedelta64(5, 'm'), 23.0),
             (datetime64(2014, 1, 1, 12, 10), numpy.timedelta64(5, 'm'), 44.0),
-        ], self.storage.get_measures(self.metric))
+        ], self.storage.get_measures(self.metric, granularities))
 
         # One year later…
-        self.incoming.add_measures(self.metric, [
+        self.incoming.add_measures(self.metric.id, [
             incoming.Measure(datetime64(2015, 1, 1, 12, 0, 1), 69),
         ])
         self.trigger_processing()
 
         self.assertEqual([
-            (datetime64(2014, 1, 1), numpy.timedelta64(1, 'D'), 39.75),
             (datetime64(2015, 1, 1), numpy.timedelta64(1, 'D'), 69),
             (datetime64(2015, 1, 1, 12), numpy.timedelta64(1, 'h'), 69),
             (datetime64(2015, 1, 1, 12), numpy.timedelta64(5, 'm'), 69),
-        ], self.storage.get_measures(self.metric))
+        ], self.storage.get_measures(self.metric, granularities))
 
         self.assertEqual({
             carbonara.SplitKey(numpy.datetime64(1244160000, 's'),
@@ -305,7 +329,7 @@ class TestStorageDriver(tests_base.TestCase):
                                  apname)
 
         # First store some points scattered across different splits
-        self.incoming.add_measures(self.metric, [
+        self.incoming.add_measures(self.metric.id, [
             incoming.Measure(datetime64(2016, 1, 1, 12, 0, 1), 69),
             incoming.Measure(datetime64(2016, 1, 2, 13, 7, 31), 42),
             incoming.Measure(datetime64(2016, 1, 4, 14, 9, 31), 4),
@@ -329,22 +353,22 @@ class TestStorageDriver(tests_base.TestCase):
             assertCompressedIfWriteFull = self.assertFalse
 
         data = self.storage._get_measures(
-            self.metric, carbonara.SplitKey(
+            self.metric, [carbonara.SplitKey(
                 numpy.datetime64(1451520000, 's'),
                 numpy.timedelta64(1, 'm'),
-            ), "mean")
+            )], "mean")[0]
         self.assertTrue(carbonara.AggregatedTimeSerie.is_compressed(data))
         data = self.storage._get_measures(
-            self.metric, carbonara.SplitKey(
+            self.metric, [carbonara.SplitKey(
                 numpy.datetime64(1451736000, 's'),
                 numpy.timedelta64(60, 's'),
-            ), "mean")
+            )], "mean")[0]
         self.assertTrue(carbonara.AggregatedTimeSerie.is_compressed(data))
         data = self.storage._get_measures(
-            self.metric, carbonara.SplitKey(
+            self.metric, [carbonara.SplitKey(
                 numpy.datetime64(1451952000, 's'),
                 numpy.timedelta64(60, 's'),
-            ), "mean")
+            )], "mean")[0]
         assertCompressedIfWriteFull(
             carbonara.AggregatedTimeSerie.is_compressed(data))
 
@@ -353,14 +377,15 @@ class TestStorageDriver(tests_base.TestCase):
             (datetime64(2016, 1, 2, 13, 7), numpy.timedelta64(1, 'm'), 42),
             (datetime64(2016, 1, 4, 14, 9), numpy.timedelta64(1, 'm'), 4),
             (datetime64(2016, 1, 6, 15, 12), numpy.timedelta64(1, 'm'), 44),
-        ], self.storage.get_measures(self.metric,
-                                     granularity=numpy.timedelta64(1, 'm')))
+        ], self.storage.get_measures(
+            self.metric,
+            granularities=[numpy.timedelta64(1, 'm')]))
 
         # Now store brand new points that should force a rewrite of one of the
         # split (keep in mind the back window size in one hour here). We move
         # the BoundTimeSerie processing timeserie far away from its current
         # range.
-        self.incoming.add_measures(self.metric, [
+        self.incoming.add_measures(self.metric.id, [
             incoming.Measure(datetime64(2016, 1, 10, 16, 18, 45), 45),
             incoming.Measure(datetime64(2016, 1, 10, 17, 12, 45), 46),
         ])
@@ -378,29 +403,29 @@ class TestStorageDriver(tests_base.TestCase):
         }, self.storage._list_split_keys_for_metric(
             self.metric, "mean", numpy.timedelta64(1, 'm')))
         data = self.storage._get_measures(
-            self.metric, carbonara.SplitKey(
+            self.metric, [carbonara.SplitKey(
                 numpy.datetime64(1451520000, 's'),
                 numpy.timedelta64(60, 's'),
-            ), "mean")
+            )], "mean")[0]
         self.assertTrue(carbonara.AggregatedTimeSerie.is_compressed(data))
         data = self.storage._get_measures(
-            self.metric, carbonara.SplitKey(
+            self.metric, [carbonara.SplitKey(
                 numpy.datetime64(1451736000, 's'),
                 numpy.timedelta64(60, 's'),
-            ), "mean")
+            )], "mean")[0]
         self.assertTrue(carbonara.AggregatedTimeSerie.is_compressed(data))
         data = self.storage._get_measures(
-            self.metric, carbonara.SplitKey(
+            self.metric, [carbonara.SplitKey(
                 numpy.datetime64(1451952000, 's'),
                 numpy.timedelta64(1, 'm'),
-            ), "mean")
+            )], "mean")[0]
         # Now this one is compressed because it has been rewritten!
         self.assertTrue(carbonara.AggregatedTimeSerie.is_compressed(data))
         data = self.storage._get_measures(
-            self.metric, carbonara.SplitKey(
+            self.metric, [carbonara.SplitKey(
                 numpy.datetime64(1452384000, 's'),
                 numpy.timedelta64(60, 's'),
-            ), "mean")
+            )], "mean")[0]
         assertCompressedIfWriteFull(
             carbonara.AggregatedTimeSerie.is_compressed(data))
 
@@ -411,8 +436,9 @@ class TestStorageDriver(tests_base.TestCase):
             (datetime64(2016, 1, 6, 15, 12), numpy.timedelta64(1, 'm'), 44),
             (datetime64(2016, 1, 10, 16, 18), numpy.timedelta64(1, 'm'), 45),
             (datetime64(2016, 1, 10, 17, 12), numpy.timedelta64(1, 'm'), 46),
-        ], self.storage.get_measures(self.metric,
-                                     granularity=numpy.timedelta64(1, 'm')))
+        ], self.storage.get_measures(
+            self.metric,
+            granularities=[numpy.timedelta64(1, 'm')]))
 
     def test_rewrite_measures_oldest_mutable_timestamp_eq_next_key(self):
         """See LP#1655422"""
@@ -426,7 +452,7 @@ class TestStorageDriver(tests_base.TestCase):
                                  apname)
 
         # First store some points scattered across different splits
-        self.incoming.add_measures(self.metric, [
+        self.incoming.add_measures(self.metric.id, [
             incoming.Measure(datetime64(2016, 1, 1, 12, 0, 1), 69),
             incoming.Measure(datetime64(2016, 1, 2, 13, 7, 31), 42),
             incoming.Measure(datetime64(2016, 1, 4, 14, 9, 31), 4),
@@ -450,22 +476,22 @@ class TestStorageDriver(tests_base.TestCase):
             assertCompressedIfWriteFull = self.assertFalse
 
         data = self.storage._get_measures(
-            self.metric, carbonara.SplitKey(
+            self.metric, [carbonara.SplitKey(
                 numpy.datetime64(1451520000, 's'),
                 numpy.timedelta64(1, 'm'),
-            ), "mean")
+            )], "mean")[0]
         self.assertTrue(carbonara.AggregatedTimeSerie.is_compressed(data))
         data = self.storage._get_measures(
-            self.metric, carbonara.SplitKey(
+            self.metric, [carbonara.SplitKey(
                 numpy.datetime64(1451736000, 's'),
                 numpy.timedelta64(1, 'm'),
-            ), "mean")
+            )], "mean")[0]
         self.assertTrue(carbonara.AggregatedTimeSerie.is_compressed(data))
         data = self.storage._get_measures(
-            self.metric, carbonara.SplitKey(
+            self.metric, [carbonara.SplitKey(
                 numpy.datetime64(1451952000, 's'),
                 numpy.timedelta64(1, 'm')
-            ), "mean")
+            )], "mean")[0]
         assertCompressedIfWriteFull(
             carbonara.AggregatedTimeSerie.is_compressed(data))
 
@@ -474,8 +500,9 @@ class TestStorageDriver(tests_base.TestCase):
             (datetime64(2016, 1, 2, 13, 7), numpy.timedelta64(1, 'm'), 42),
             (datetime64(2016, 1, 4, 14, 9), numpy.timedelta64(1, 'm'), 4),
             (datetime64(2016, 1, 6, 15, 12), numpy.timedelta64(1, 'm'), 44),
-        ], self.storage.get_measures(self.metric,
-                                     granularity=numpy.timedelta64(60, 's')))
+        ], self.storage.get_measures(
+            self.metric,
+            granularities=[numpy.timedelta64(60, 's')]))
 
         # Now store brand new points that should force a rewrite of one of the
         # split (keep in mind the back window size in one hour here). We move
@@ -484,7 +511,7 @@ class TestStorageDriver(tests_base.TestCase):
 
         # Here we test a special case where the oldest_mutable_timestamp will
         # be 2016-01-10TOO:OO:OO = 1452384000.0, our new split key.
-        self.incoming.add_measures(self.metric, [
+        self.incoming.add_measures(self.metric.id, [
             incoming.Measure(datetime64(2016, 1, 10, 0, 12), 45),
         ])
         self.trigger_processing()
@@ -501,29 +528,29 @@ class TestStorageDriver(tests_base.TestCase):
         }, self.storage._list_split_keys_for_metric(
             self.metric, "mean", numpy.timedelta64(1, 'm')))
         data = self.storage._get_measures(
-            self.metric, carbonara.SplitKey(
+            self.metric, [carbonara.SplitKey(
                 numpy.datetime64(1451520000, 's'),
                 numpy.timedelta64(1, 'm'),
-            ), "mean")
+            )], "mean")[0]
         self.assertTrue(carbonara.AggregatedTimeSerie.is_compressed(data))
         data = self.storage._get_measures(
-            self.metric, carbonara.SplitKey(
+            self.metric, [carbonara.SplitKey(
                 numpy.datetime64(1451736000, 's'),
                 numpy.timedelta64(1, 'm'),
-            ), "mean")
+            )], "mean")[0]
         self.assertTrue(carbonara.AggregatedTimeSerie.is_compressed(data))
         data = self.storage._get_measures(
-            self.metric, carbonara.SplitKey(
+            self.metric, [carbonara.SplitKey(
                 numpy.datetime64(1451952000, 's'),
                 numpy.timedelta64(60, 's')
-            ), "mean")
+            )], "mean")[0]
         # Now this one is compressed because it has been rewritten!
         self.assertTrue(carbonara.AggregatedTimeSerie.is_compressed(data))
         data = self.storage._get_measures(
-            self.metric, carbonara.SplitKey(
+            self.metric, [carbonara.SplitKey(
                 numpy.datetime64(1452384000, 's'),
                 numpy.timedelta64(1, 'm'),
-            ), "mean")
+            )], "mean")[0]
         assertCompressedIfWriteFull(
             carbonara.AggregatedTimeSerie.is_compressed(data))
 
@@ -533,8 +560,9 @@ class TestStorageDriver(tests_base.TestCase):
             (datetime64(2016, 1, 4, 14, 9), numpy.timedelta64(1, 'm'), 4),
             (datetime64(2016, 1, 6, 15, 12), numpy.timedelta64(1, 'm'), 44),
             (datetime64(2016, 1, 10, 0, 12), numpy.timedelta64(1, 'm'), 45),
-        ], self.storage.get_measures(self.metric,
-                                     granularity=numpy.timedelta64(60, 's')))
+        ], self.storage.get_measures(
+            self.metric,
+            granularities=[numpy.timedelta64(60, 's')]))
 
     def test_rewrite_measures_corruption_missing_file(self):
         # Create an archive policy that spans on several splits. Each split
@@ -547,7 +575,7 @@ class TestStorageDriver(tests_base.TestCase):
                                  apname)
 
         # First store some points scattered across different splits
-        self.incoming.add_measures(self.metric, [
+        self.incoming.add_measures(self.metric.id, [
             incoming.Measure(datetime64(2016, 1, 1, 12, 0, 1), 69),
             incoming.Measure(datetime64(2016, 1, 2, 13, 7, 31), 42),
             incoming.Measure(datetime64(2016, 1, 4, 14, 9, 31), 4),
@@ -572,22 +600,22 @@ class TestStorageDriver(tests_base.TestCase):
 
         data = self.storage._get_measures(
             self.metric,
-            carbonara.SplitKey(
+            [carbonara.SplitKey(
                 numpy.datetime64(1451520000, 's'),
                 numpy.timedelta64(1, 'm'),
-            ), "mean")
+            )], "mean")[0]
         self.assertTrue(carbonara.AggregatedTimeSerie.is_compressed(data))
         data = self.storage._get_measures(
-            self.metric, carbonara.SplitKey(
+            self.metric, [carbonara.SplitKey(
                 numpy.datetime64(1451736000, 's'),
                 numpy.timedelta64(1, 'm')
-            ), "mean")
+            )], "mean")[0]
         self.assertTrue(carbonara.AggregatedTimeSerie.is_compressed(data))
         data = self.storage._get_measures(
-            self.metric, carbonara.SplitKey(
+            self.metric, [carbonara.SplitKey(
                 numpy.datetime64(1451952000, 's'),
                 numpy.timedelta64(1, 'm'),
-            ), "mean")
+            )], "mean")[0]
         assertCompressedIfWriteFull(
             carbonara.AggregatedTimeSerie.is_compressed(data))
 
@@ -600,8 +628,9 @@ class TestStorageDriver(tests_base.TestCase):
              numpy.timedelta64(1, 'm'), 4),
             (datetime64(2016, 1, 6, 15, 12),
              numpy.timedelta64(1, 'm'), 44),
-        ], self.storage.get_measures(self.metric,
-                                     granularity=numpy.timedelta64(60, 's')))
+        ], self.storage.get_measures(
+            self.metric,
+            granularities=[numpy.timedelta64(60, 's')]))
 
         # Test what happens if we delete the latest split and then need to
         # compress it!
@@ -615,7 +644,7 @@ class TestStorageDriver(tests_base.TestCase):
         # split (keep in mind the back window size in one hour here). We move
         # the BoundTimeSerie processing timeserie far away from its current
         # range.
-        self.incoming.add_measures(self.metric, [
+        self.incoming.add_measures(self.metric.id, [
             incoming.Measure(datetime64(2016, 1, 10, 16, 18, 45), 45),
             incoming.Measure(datetime64(2016, 1, 10, 17, 12, 45), 46),
         ])
@@ -632,7 +661,7 @@ class TestStorageDriver(tests_base.TestCase):
                                  apname)
 
         # First store some points scattered across different splits
-        self.incoming.add_measures(self.metric, [
+        self.incoming.add_measures(self.metric.id, [
             incoming.Measure(datetime64(2016, 1, 1, 12, 0, 1), 69),
             incoming.Measure(datetime64(2016, 1, 2, 13, 7, 31), 42),
             incoming.Measure(datetime64(2016, 1, 4, 14, 9, 31), 4),
@@ -656,22 +685,22 @@ class TestStorageDriver(tests_base.TestCase):
             assertCompressedIfWriteFull = self.assertFalse
 
         data = self.storage._get_measures(
-            self.metric, carbonara.SplitKey(
+            self.metric, [carbonara.SplitKey(
                 numpy.datetime64(1451520000, 's'),
                 numpy.timedelta64(60, 's'),
-            ), "mean")
+            )], "mean")[0]
         self.assertTrue(carbonara.AggregatedTimeSerie.is_compressed(data))
         data = self.storage._get_measures(
-            self.metric, carbonara.SplitKey(
+            self.metric, [carbonara.SplitKey(
                 numpy.datetime64(1451736000, 's'),
                 numpy.timedelta64(1, 'm'),
-            ), "mean")
+            )], "mean")[0]
         self.assertTrue(carbonara.AggregatedTimeSerie.is_compressed(data))
         data = self.storage._get_measures(
-            self.metric, carbonara.SplitKey(
+            self.metric, [carbonara.SplitKey(
                 numpy.datetime64(1451952000, 's'),
                 numpy.timedelta64(1, 'm'),
-            ), "mean")
+            )], "mean")[0]
         assertCompressedIfWriteFull(
             carbonara.AggregatedTimeSerie.is_compressed(data))
 
@@ -680,8 +709,9 @@ class TestStorageDriver(tests_base.TestCase):
             (datetime64(2016, 1, 2, 13, 7), numpy.timedelta64(1, 'm'), 42),
             (datetime64(2016, 1, 4, 14, 9), numpy.timedelta64(1, 'm'), 4),
             (datetime64(2016, 1, 6, 15, 12), numpy.timedelta64(1, 'm'), 44),
-        ], self.storage.get_measures(self.metric,
-                                     granularity=numpy.timedelta64(1, 'm')))
+        ], self.storage.get_measures(
+            self.metric,
+            granularities=[numpy.timedelta64(1, 'm')]))
 
         # Test what happens if we write garbage
         self.storage._store_metric_measures(
@@ -695,27 +725,33 @@ class TestStorageDriver(tests_base.TestCase):
         # split (keep in mind the back window size in one hour here). We move
         # the BoundTimeSerie processing timeserie far away from its current
         # range.
-        self.incoming.add_measures(self.metric, [
+        self.incoming.add_measures(self.metric.id, [
             incoming.Measure(datetime64(2016, 1, 10, 16, 18, 45), 45),
             incoming.Measure(datetime64(2016, 1, 10, 17, 12, 45), 46),
         ])
         self.trigger_processing()
 
     def test_updated_measures(self):
-        self.incoming.add_measures(self.metric, [
+        self.incoming.add_measures(self.metric.id, [
             incoming.Measure(datetime64(2014, 1, 1, 12, 0, 1), 69),
             incoming.Measure(datetime64(2014, 1, 1, 12, 7, 31), 42),
         ])
         self.trigger_processing()
 
+        granularities = [
+            numpy.timedelta64(1, 'D'),
+            numpy.timedelta64(1, 'h'),
+            numpy.timedelta64(5, 'm'),
+        ]
+
         self.assertEqual([
             (datetime64(2014, 1, 1), numpy.timedelta64(1, 'D'), 55.5),
             (datetime64(2014, 1, 1, 12), numpy.timedelta64(1, 'h'), 55.5),
             (datetime64(2014, 1, 1, 12), numpy.timedelta64(5, 'm'), 69),
             (datetime64(2014, 1, 1, 12, 5), numpy.timedelta64(5, 'm'), 42.0),
-        ], self.storage.get_measures(self.metric))
+        ], self.storage.get_measures(self.metric, granularities))
 
-        self.incoming.add_measures(self.metric, [
+        self.incoming.add_measures(self.metric.id, [
             incoming.Measure(datetime64(2014, 1, 1, 12, 9, 31), 4),
             incoming.Measure(datetime64(2014, 1, 1, 12, 12, 45), 44),
         ])
@@ -727,7 +763,7 @@ class TestStorageDriver(tests_base.TestCase):
             (datetime64(2014, 1, 1, 12), numpy.timedelta64(5, 'm'), 69.0),
             (datetime64(2014, 1, 1, 12, 5), numpy.timedelta64(5, 'm'), 23.0),
             (datetime64(2014, 1, 1, 12, 10), numpy.timedelta64(5, 'm'), 44.0),
-        ], self.storage.get_measures(self.metric))
+        ], self.storage.get_measures(self.metric, granularities))
 
         self.assertEqual([
             (datetime64(2014, 1, 1), numpy.timedelta64(1, 'D'), 69),
@@ -735,7 +771,8 @@ class TestStorageDriver(tests_base.TestCase):
             (datetime64(2014, 1, 1, 12), numpy.timedelta64(5, 'm'), 69.0),
             (datetime64(2014, 1, 1, 12, 5), numpy.timedelta64(5, 'm'), 42.0),
             (datetime64(2014, 1, 1, 12, 10), numpy.timedelta64(5, 'm'), 44.0),
-        ], self.storage.get_measures(self.metric, aggregation='max'))
+        ], self.storage.get_measures(self.metric,
+                                     granularities, aggregation='max'))
 
         self.assertEqual([
             (datetime64(2014, 1, 1), numpy.timedelta64(1, 'D'), 4),
@@ -743,10 +780,11 @@ class TestStorageDriver(tests_base.TestCase):
             (datetime64(2014, 1, 1, 12), numpy.timedelta64(5, 'm'), 69.0),
             (datetime64(2014, 1, 1, 12, 5), numpy.timedelta64(5, 'm'), 4.0),
             (datetime64(2014, 1, 1, 12, 10), numpy.timedelta64(5, 'm'), 44.0),
-        ], self.storage.get_measures(self.metric, aggregation='min'))
+        ], self.storage.get_measures(self.metric,
+                                     granularities, aggregation='min'))
 
     def test_add_and_get_measures(self):
-        self.incoming.add_measures(self.metric, [
+        self.incoming.add_measures(self.metric.id, [
             incoming.Measure(datetime64(2014, 1, 1, 12, 0, 1), 69),
             incoming.Measure(datetime64(2014, 1, 1, 12, 7, 31), 42),
             incoming.Measure(datetime64(2014, 1, 1, 12, 9, 31), 4),
@@ -754,13 +792,19 @@ class TestStorageDriver(tests_base.TestCase):
         ])
         self.trigger_processing()
 
+        granularities = [
+            numpy.timedelta64(1, 'D'),
+            numpy.timedelta64(1, 'h'),
+            numpy.timedelta64(5, 'm'),
+        ]
+
         self.assertEqual([
             (datetime64(2014, 1, 1), numpy.timedelta64(1, 'D'), 39.75),
             (datetime64(2014, 1, 1, 12), numpy.timedelta64(1, 'h'), 39.75),
             (datetime64(2014, 1, 1, 12), numpy.timedelta64(5, 'm'), 69.0),
             (datetime64(2014, 1, 1, 12, 5), numpy.timedelta64(5, 'm'), 23.0),
             (datetime64(2014, 1, 1, 12, 10), numpy.timedelta64(5, 'm'), 44.0),
-        ], self.storage.get_measures(self.metric))
+        ], self.storage.get_measures(self.metric, granularities))
 
         self.assertEqual([
             (datetime64(2014, 1, 1), numpy.timedelta64(1, 'D'), 39.75),
@@ -768,6 +812,7 @@ class TestStorageDriver(tests_base.TestCase):
             (datetime64(2014, 1, 1, 12, 10), numpy.timedelta64(5, 'm'), 44.0),
         ], self.storage.get_measures(
             self.metric,
+            granularities,
             from_timestamp=datetime64(2014, 1, 1, 12, 10, 0)))
 
         self.assertEqual([
@@ -777,6 +822,7 @@ class TestStorageDriver(tests_base.TestCase):
             (datetime64(2014, 1, 1, 12, 5), numpy.timedelta64(5, 'm'), 23.0),
         ], self.storage.get_measures(
             self.metric,
+            granularities,
             to_timestamp=datetime64(2014, 1, 1, 12, 6, 0)))
 
         self.assertEqual([
@@ -785,6 +831,7 @@ class TestStorageDriver(tests_base.TestCase):
             (datetime64(2014, 1, 1, 12, 10), numpy.timedelta64(5, 'm'), 44.0),
         ], self.storage.get_measures(
             self.metric,
+            granularities,
             to_timestamp=datetime64(2014, 1, 1, 12, 10, 10),
             from_timestamp=datetime64(2014, 1, 1, 12, 10, 10)))
 
@@ -794,6 +841,7 @@ class TestStorageDriver(tests_base.TestCase):
             (datetime64(2014, 1, 1, 12), numpy.timedelta64(5, 'm'), 69.0),
         ], self.storage.get_measures(
             self.metric,
+            granularities,
             from_timestamp=datetime64(2014, 1, 1, 12, 0, 0),
             to_timestamp=datetime64(2014, 1, 1, 12, 0, 2)))
 
@@ -803,6 +851,7 @@ class TestStorageDriver(tests_base.TestCase):
             (datetime64(2014, 1, 1, 12), numpy.timedelta64(5, 'm'), 69.0),
         ], self.storage.get_measures(
             self.metric,
+            granularities,
             from_timestamp=datetime64(2014, 1, 1, 12),
             to_timestamp=datetime64(2014, 1, 1, 12, 0, 2)))
 
@@ -812,7 +861,7 @@ class TestStorageDriver(tests_base.TestCase):
             self.metric,
             from_timestamp=datetime64(2014, 1, 1, 12, 0, 0),
             to_timestamp=datetime64(2014, 1, 1, 12, 0, 2),
-            granularity=numpy.timedelta64(1, 'h')))
+            granularities=[numpy.timedelta64(1, 'h')]))
 
         self.assertEqual([
             (datetime64(2014, 1, 1, 12), numpy.timedelta64(5, 'm'), 69.0),
@@ -820,27 +869,32 @@ class TestStorageDriver(tests_base.TestCase):
             self.metric,
             from_timestamp=datetime64(2014, 1, 1, 12, 0, 0),
             to_timestamp=datetime64(2014, 1, 1, 12, 0, 2),
-            granularity=numpy.timedelta64(5, 'm')))
+            granularities=[numpy.timedelta64(5, 'm')]))
 
-        self.assertRaises(storage.GranularityDoesNotExist,
+        self.assertRaises(storage.AggregationDoesNotExist,
                           self.storage.get_measures,
                           self.metric,
-                          granularity=numpy.timedelta64(42, 's'))
+                          granularities=[numpy.timedelta64(42, 's')])
 
     def test_get_measure_unknown_aggregation(self):
-        self.incoming.add_measures(self.metric, [
+        self.incoming.add_measures(self.metric.id, [
             incoming.Measure(datetime64(2014, 1, 1, 12, 0, 1), 69),
             incoming.Measure(datetime64(2014, 1, 1, 12, 7, 31), 42),
             incoming.Measure(datetime64(2014, 1, 1, 12, 9, 31), 4),
             incoming.Measure(datetime64(2014, 1, 1, 12, 12, 45), 44),
         ])
-        self.assertRaises(storage.AggregationDoesNotExist,
-                          self.storage.get_measures,
-                          self.metric, aggregation='last')
+        granularities = [
+            numpy.timedelta64(1, 'D'),
+            numpy.timedelta64(1, 'h'),
+            numpy.timedelta64(5, 'm'),
+        ]
+        self.assertEqual(
+            [], self.storage.get_measures(
+                self.metric, granularities, aggregation='last'))
 
-    def test_search_value(self):
+    def test_find_measures(self):
         metric2, __ = self._create_metric()
-        self.incoming.add_measures(self.metric, [
+        self.incoming.add_measures(self.metric.id, [
             incoming.Measure(datetime64(2014, 1, 1, 12, 0, 1,), 69),
             incoming.Measure(datetime64(2014, 1, 1, 12, 7, 31), 42),
             incoming.Measure(datetime64(2014, 1, 1, 12, 5, 31), 8),
@@ -848,7 +902,7 @@ class TestStorageDriver(tests_base.TestCase):
             incoming.Measure(datetime64(2014, 1, 1, 12, 12, 45), 42),
         ])
 
-        self.incoming.add_measures(metric2, [
+        self.incoming.add_measures(metric2.id, [
             incoming.Measure(datetime64(2014, 1, 1, 12, 0, 5), 9),
             incoming.Measure(datetime64(2014, 1, 1, 12, 7, 31), 2),
             incoming.Measure(datetime64(2014, 1, 1, 12, 9, 31), 6),
@@ -857,35 +911,54 @@ class TestStorageDriver(tests_base.TestCase):
         self.trigger_processing([str(self.metric.id), str(metric2.id)])
 
         self.assertEqual(
-            {metric2: [],
-             self.metric: [
-                 (datetime64(2014, 1, 1),
-                  numpy.timedelta64(1, 'D'), 33),
-                 (datetime64(2014, 1, 1, 12),
-                  numpy.timedelta64(1, 'h'), 33),
-                 (datetime64(2014, 1, 1, 12),
-                  numpy.timedelta64(5, 'm'), 69),
-                 (datetime64(2014, 1, 1, 12, 10),
-                  numpy.timedelta64(5, 'm'), 42)]},
-            self.storage.search_value(
-                [metric2, self.metric],
-                {u"≥": 30}))
+            [
+                (datetime64(2014, 1, 1),
+                 numpy.timedelta64(1, 'D'), 33),
+            ],
+            self.storage.find_measure(
+                self.metric, storage.MeasureQuery({u"≥": 30}),
+                numpy.timedelta64(1, 'D')))
 
         self.assertEqual(
-            {metric2: [], self.metric: []},
-            self.storage.search_value(
-                [metric2, self.metric],
-                {u"∧": [
+            [
+                (datetime64(2014, 1, 1, 12),
+                 numpy.timedelta64(5, 'm'), 69),
+                (datetime64(2014, 1, 1, 12, 10),
+                 numpy.timedelta64(5, 'm'), 42)
+            ],
+            self.storage.find_measure(
+                self.metric, storage.MeasureQuery({u"≥": 30}),
+                numpy.timedelta64(5, 'm')))
+
+        self.assertEqual(
+            [],
+            self.storage.find_measure(
+                metric2, storage.MeasureQuery({u"≥": 30}),
+                numpy.timedelta64(5, 'm')))
+
+        self.assertEqual(
+            [],
+            self.storage.find_measure(
+                self.metric, storage.MeasureQuery({u"∧": [
                     {u"eq": 100},
-                    {u"≠": 50}]}))
+                    {u"≠": 50}]}),
+                numpy.timedelta64(5, 'm')))
+
+        self.assertEqual(
+            [],
+            self.storage.find_measure(
+                metric2, storage.MeasureQuery({u"∧": [
+                    {u"eq": 100},
+                    {u"≠": 50}]}),
+                numpy.timedelta64(5, 'm')))
 
     def test_resize_policy(self):
         name = str(uuid.uuid4())
         ap = archive_policy.ArchivePolicy(name, 0, [(3, 5)])
         self.index.create_archive_policy(ap)
         m = self.index.create_metric(uuid.uuid4(), str(uuid.uuid4()), name)
-        m = self.index.list_metrics(ids=[m.id])[0]
-        self.incoming.add_measures(m, [
+        m = self.index.list_metrics(attribute_filter={"=": {"id": m.id}})[0]
+        self.incoming.add_measures(m.id, [
             incoming.Measure(datetime64(2014, 1, 1, 12, 0, 0), 1),
             incoming.Measure(datetime64(2014, 1, 1, 12, 0, 5), 1),
             incoming.Measure(datetime64(2014, 1, 1, 12, 0, 10), 1),
@@ -895,12 +968,12 @@ class TestStorageDriver(tests_base.TestCase):
             (datetime64(2014, 1, 1, 12, 0, 0), numpy.timedelta64(5, 's'), 1),
             (datetime64(2014, 1, 1, 12, 0, 5), numpy.timedelta64(5, 's'), 1),
             (datetime64(2014, 1, 1, 12, 0, 10), numpy.timedelta64(5, 's'), 1),
-        ], self.storage.get_measures(m))
+        ], self.storage.get_measures(m, [numpy.timedelta64(5, 's')]))
         # expand to more points
         self.index.update_archive_policy(
             name, [archive_policy.ArchivePolicyItem(granularity=5, points=6)])
-        m = self.index.list_metrics(ids=[m.id])[0]
-        self.incoming.add_measures(m, [
+        m = self.index.list_metrics(attribute_filter={"=": {"id": m.id}})[0]
+        self.incoming.add_measures(m.id, [
             incoming.Measure(datetime64(2014, 1, 1, 12, 0, 15), 1),
         ])
         self.trigger_processing([str(m.id)])
@@ -909,24 +982,24 @@ class TestStorageDriver(tests_base.TestCase):
             (datetime64(2014, 1, 1, 12, 0, 5), numpy.timedelta64(5, 's'), 1),
             (datetime64(2014, 1, 1, 12, 0, 10), numpy.timedelta64(5, 's'), 1),
             (datetime64(2014, 1, 1, 12, 0, 15), numpy.timedelta64(5, 's'), 1),
-        ], self.storage.get_measures(m))
+        ], self.storage.get_measures(m, [numpy.timedelta64(5, 's')]))
         # shrink timespan
         self.index.update_archive_policy(
             name, [archive_policy.ArchivePolicyItem(granularity=5, points=2)])
-        m = self.index.list_metrics(ids=[m.id])[0]
+        m = self.index.list_metrics(attribute_filter={"=": {"id": m.id}})[0]
         self.assertEqual([
             (datetime64(2014, 1, 1, 12, 0, 10), numpy.timedelta64(5, 's'), 1),
             (datetime64(2014, 1, 1, 12, 0, 15), numpy.timedelta64(5, 's'), 1),
-        ], self.storage.get_measures(m))
+        ], self.storage.get_measures(m, [numpy.timedelta64(5, 's')]))
 
     def test_resample_no_metric(self):
         """https://github.com/gnocchixyz/gnocchi/issues/69"""
         self.assertEqual([],
                          self.storage.get_measures(
                              self.metric,
+                             [numpy.timedelta64(300, 's')],
                              datetime64(2014, 1, 1),
                              datetime64(2015, 1, 1),
-                             granularity=numpy.timedelta64(300, 's'),
                              resample=numpy.timedelta64(1, 'h')))
 
 
